#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Test for taurus.core.util.timer"""

#__all__ = []

__docformat__ = 'restructuredtext'

import time
import threading
import numpy
from taurus.external import unittest
from taurus.core.util.timer import Timer


class TimerTest(unittest.TestCase):
    '''Test case for testing the taurus.core.util.timer.Timer class'''

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.__count = 0
        self.__calltimes = []

    def tearDown(self):
        self.__count = 0
        self.__calltimes = []

    def test_calltimes(self):
        '''check the quality of the Timer's timing'''
        period = .1  # in s
        n = 10  # make this >2 !
        timeout = n * period + 2  # expected time +2 seconds margin
        tol = 0.001  # time tolerance for the various checks (in s)

        timer = Timer(period, self._callback, None, strict_timing=True,
                      sleep=.05, n=n)

        # Start the timer, wait till the callback is called n times and then s
        # and then stop it
        self.__nCalls = threading.Event()
        timer.start()
        self.__nCalls.wait(timeout)
        timer.stop()
        self.__nCalls.clear()

        # checking number of calls
        ts = numpy.array(self.__calltimes)
        msg = '%i calls expected (got %i)' % (n, ts.size)
        self.assertEqual(ts.size, n, msg)

        # checking drift
        totaltime = ts[-1] - ts[0]
        drift = abs(totaltime - ((n - 1) * period))
        msg = 'Too much drift (%g). Tolerance=%g' % (drift, tol)
        self.assertLess(drift, tol, msg)

        # checking period jitter (mean period and its standard dev)
        periods = numpy.diff(ts)
        mean = periods.mean()
        std = periods.std()
        msg = 'Wrong period. Expected: %g +/- %g Got: %g +/- %g' % (period, tol,
                                                                    mean, std)
        self.assertAlmostEqual(mean, period, msg=msg, delta=tol)
        self.assertLess(std, tol, msg)

    def _callback(self, sleep=0, n=5):
        '''store times at which it has been called, and signal when n calls
        have been done. If sleep>0 is passed, sleep that much in each call '''
        self.__calltimes.append(time.time())
        self.__count += 1
        time.sleep(sleep)
        if self.__count == n:
            self.__nCalls.set()  # signal that we have been called n times


if __name__ == '__main__':
    pass



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
qsevensegmentdisplay.py
"""

__all__ = ['Q7SegDigit']

__docformat__ = 'restructuredtext'

from taurus.external.qt import Qt

POLY = Qt.QPolygonF
P = Qt.QPointF


class Q7SegDigit(Qt.QWidget):
    """
    A widget representing a single seven segment digit. The style can be
    configured through the widget properties. For example, a typical LCD would
    have the following style:

        - bgColor 170, 170, 127
        - ledOnPenColor 30,30,30
        - ledOnBgColor 0,0,0
        - ledOffPenColor 160, 160, 120
        - ledOffbgColor 150, 150, 112
    """
    Leds = (
        (1, 1, 1, 0, 1, 1, 1),  # 0
        (0, 0, 1, 0, 0, 1, 0),  # 1
        (1, 0, 1, 1, 1, 0, 1),  # 2
        (1, 0, 1, 1, 0, 1, 1),  # 3
        (0, 1, 1, 1, 0, 1, 0),  # 4
        (1, 1, 0, 1, 0, 1, 1),  # 5
        (1, 1, 0, 1, 1, 1, 1),  # 6
        (1, 0, 1, 0, 0, 1, 0),  # 7
        (1, 1, 1, 1, 1, 1, 1),  # 8
        (1, 1, 1, 1, 0, 1, 0),  # 9
        (0, 0, 0, 0, 0, 0, 0),  # 10 'nothing'
        (0, 0, 0, 1, 0, 0, 0),  # 11 -
        (1, 0, 1, 1, 1, 1, 1),  # 12 a
        (0, 0, 0, 1, 1, 1, 0),  # 13 n
    )

#    LedGeometries = (
#        POLY([P(67, 60), P(283, 60), P(253, 90), P(97, 90), P(67, 60)]),
#        POLY([P(60, 67), P(90, 97), P(90, 178), P(60, 193), P(60, 67)]),
#        POLY([P(290, 67), P(260, 97), P(260, 178), P(290, 193), P(290, 67)]),
#        POLY([P(67, 200), P(97, 185), P(253, 185), P(283, 200), P(253, 215), P(97, 215), P(67, 200)]),
#        POLY([P(60, 207), P(90, 222), P(90, 303), P(60, 333), P(60, 207)]),
#        POLY([P(290, 207), P(260, 222), P(260, 303), P(290, 333), P(290, 207)]),
#        POLY([P(67, 340), P(283, 340), P(253, 310), P(97, 310), P(67, 340)]),
#    )

    LedGeometriesWithFrame300x400 = (
        POLY([P(37, 30), P(263, 30), P(233, 60), P(67, 60), P(37, 30)]),
        POLY([P(30, 37), P(60, 67), P(60, 178), P(30, 193), P(30, 37)]),
        POLY([P(270, 37), P(240, 67), P(240, 178), P(270, 193), P(270, 37)]),
        POLY([P(37, 200), P(67, 185), P(233, 185), P(
            263, 200), P(233, 215), P(67, 215), P(37, 200)]),
        POLY([P(30, 207), P(60, 222), P(60, 333), P(30, 363), P(30, 207)]),
        POLY([P(270, 207), P(240, 222), P(240, 333), P(270, 363), P(270, 207)]),
        POLY([P(37, 370), P(263, 370), P(233, 340), P(67, 340), P(37, 370)]),
    )

    LedGeometriesWithoutFrame300x400 = (
        POLY([P(7, 0), P(293, 0), P(263, 30), P(37, 30), P(7, 0)]),
        POLY([P(0, 7), P(30, 37), P(30, 178), P(0, 193), P(0, 7)]),
        POLY([P(300, 7), P(270, 37), P(270, 178), P(300, 193), P(300, 7)]),
        POLY([P(7, 200), P(37, 185), P(263, 185), P(
            293, 200), P(263, 215), P(37, 215), P(7, 200)]),
        POLY([P(0, 207), P(30, 222), P(30, 363), P(0, 393), P(0, 207)]),
        POLY([P(300, 207), P(270, 222), P(270, 363), P(300, 393), P(300, 207)]),
        POLY([P(7, 400), P(293, 400), P(263, 370), P(37, 370), P(7, 400)]),
    )

    LedGeometriesWithFrame300x300 = (
        POLY([P(37, 30), P(263, 30), P(233, 60), P(67, 60), P(37, 30)]),
        POLY([P(30, 37), P(60, 67), P(60, 128), P(30, 143), P(30, 37)]),
        POLY([P(270, 37), P(240, 67), P(240, 128), P(270, 143), P(270, 37)]),
        POLY([P(37, 150), P(67, 135), P(233, 135), P(
            263, 150), P(233, 165), P(67, 165), P(37, 150)]),
        POLY([P(30, 157), P(60, 172), P(60, 233), P(30, 263), P(30, 157)]),
        POLY([P(270, 157), P(240, 172), P(240, 233), P(270, 263), P(270, 157)]),
        POLY([P(37, 270), P(263, 270), P(233, 240), P(67, 240), P(37, 270)]),
    )

    LedGeometriesWithoutFrame300x300 = (
        POLY([P(7, 0), P(293, 0), P(263, 30), P(37, 30), P(7, 0)]),
        POLY([P(0, 7), P(30, 37), P(30, 128), P(0, 143), P(0, 7)]),
        POLY([P(300, 7), P(270, 37), P(270, 128), P(300, 143), P(300, 7)]),
        POLY([P(7, 150), P(37, 135), P(263, 135), P(
            293, 150), P(263, 165), P(37, 165), P(7, 150)]),
        POLY([P(0, 157), P(30, 172), P(30, 263), P(0, 293), P(0, 157)]),
        POLY([P(300, 157), P(270, 172), P(270, 263), P(300, 293), P(300, 157)]),
        POLY([P(7, 300), P(293, 300), P(263, 270), P(37, 270), P(7, 300)]),
    )

    DftLedOnPenColor = Qt.QColor(193, 0, 0, 255)
    DftLedOnBgColor = Qt.Qt.red

    DftLedOffPenColor = Qt.QColor(30, 30, 30, 255)
    DftLedOffBgColor = Qt.QColor(40, 40, 40, 255)

    DftBgBrush = Qt.QBrush(Qt.Qt.black, Qt.Qt.SolidPattern)

    DftLedPenWidth = 5
    DftValue = ''

    DftWidth = 300
    DftHeight = 300
    DftAspectRatio = DftWidth / DftHeight
    DftUseFrame = True

    def __init__(self, parent=None, **kwargs):
        Qt.QWidget.__init__(self, parent)

        self._setLedPenWidth(Q7SegDigit.DftLedPenWidth)
        self._setValue(Q7SegDigit.DftValue)
        self._setLedOnPenColor(Q7SegDigit.DftLedOnPenColor)
        self._setLedOnBgColor(Q7SegDigit.DftLedOnBgColor)
        self._setLedOffPenColor(Q7SegDigit.DftLedOffPenColor)
        self._setLedOffBgColor(Q7SegDigit.DftLedOffBgColor)
        self._setBgBrush(Q7SegDigit.DftBgBrush)
        self._setAspectRatio(Q7SegDigit.DftAspectRatio)
        self._setUseFrame(Q7SegDigit.DftUseFrame)
        self._updatePensAndBrushes()

    def minimumSizeHint(self):
        return Qt.QSize(4, 5.9)

    def sizeHint(self):
        # return Qt.QSize(Q7SegDigit.DftWidth, Q7SegDigit.DftHeight)
        return Qt.QSize(40, 50)

    def _updatePensAndBrushes(self):
        pon = Qt.QPen(self._ledOnPenColor, self._ledPenWidth,
                      Qt.Qt.SolidLine, Qt.Qt.RoundCap, Qt.Qt.RoundJoin)
        poff = Qt.QPen(self._ledOffPenColor, self._ledPenWidth,
                       Qt.Qt.SolidLine, Qt.Qt.RoundCap, Qt.Qt.RoundJoin)
        bon = Qt.QBrush(self._ledOnBgColor, Qt.Qt.SolidPattern)
        boff = Qt.QBrush(self._ledOffBgColor, Qt.Qt.SolidPattern)
        pens, brushes = [], []
        for nLeds in Q7SegDigit.Leds:
            nPens = []
            nBrushes = []
            for onoff in nLeds:
                if onoff:
                    pen, brush = pon, bon
                else:
                    pen, brush = poff, boff
                nPens.append(pen)
                nBrushes.append(brush)
            pens.append(nPens)
            brushes.append(nBrushes)
        self._pens, self._brushes = pens, brushes
        self.update()

    def __valueStrToLedIndex(self, s):
        if s is None:
            s = 10
        elif s == '-':
            s = 11
        elif s == 'a':
            s = 12
        elif s == 'n':
            s = 13
        else:
            try:
                s = int(s)
            except:
                s = 10
        return s

    def paintEvent(self, evt):
        painter = Qt.QPainter(self)
        painter.setRenderHint(Qt.QPainter.Antialiasing)
        painter.setWindow(0, 0, self.DftWidth, self.DftHeight)
        w, h = float(self.width()), float(self.height())
        aspect = w / h
        if aspect > 0.75:
            w = h * aspect
        else:
            h = w / aspect
        painter.setViewport(0, 0, w, h)
        self._paintBorder(painter)
        self._paintSegment(painter)

    def _paintBorder(self, painter):
        if self.getUseFrame():
            painter.setPen(Qt.QPen(Qt.Qt.black, 2, Qt.Qt.SolidLine,
                                   Qt.Qt.RoundCap, Qt.Qt.RoundJoin))
            linGrad = Qt.QLinearGradient(30, 200, 200, 150)
            linGrad.setColorAt(0, Qt.Qt.darkGray)
            linGrad.setColorAt(1, Qt.Qt.white)
            linGrad.setSpread(Qt.QGradient.ReflectSpread)
            painter.setBrush(linGrad)
            border2 = Qt.QRectF(0, 0, self.DftWidth, self.DftHeight)
            painter.drawRoundRect(border2, 10, 10)
            painter.setBrush(self.getBgBrush())
            dist = 20
            border1 = Qt.QRectF(dist, dist, self.DftWidth -
                                2 * dist, self.DftHeight - 2 * dist)
            painter.drawRoundRect(border1, 5, 5)
        else:
            painter.setBrush(self.getBgBrush())
            border1 = Qt.QRectF(0, 0, self.DftWidth, self.DftHeight)
            painter.drawRect(border1)

    def _paintSegment(self, painter):
        idx = self.__valueStrToLedIndex(self._value)

        if self.getUseFrame():
            if self.DftHeight == 300:
                geom = self.LedGeometriesWithFrame300x300
            else:
                geom = self.LedGeometriesWithFrame300x400
        else:
            if self.DftHeight == 300:
                geom = self.LedGeometriesWithoutFrame300x300
            else:
                geom = self.LedGeometriesWithoutFrame300x400

        pens, brushes = self._pens[idx], self._brushes[idx]

        for i in xrange(7):
            seg = Qt.QPainterPath()
            seg.addPolygon(geom[i])
            painter.setPen(pens[i])
            painter.setBrush(brushes[i])
            painter.drawPath(seg)

    def __str__(self):
        _, idx = '', self.__valueStrToLedIndex(self._value)
        leds = self.Leds[idx]

        # line 0
        c = ' '
        if leds[0]:
            c = '_'
        ret = ' %c \n' % c

        # line 1
        c1, c2, c3 = ' ', ' ', ' '
        if leds[1]:
            c1 = '|'
        if leds[3]:
            c2 = '_'
        if leds[2]:
            c3 = '|'
        ret += '%c%c%c\n' % (c1, c2, c3)

        # line 2
        c1, c2, c3 = ' ', ' ', ' '
        if leds[4]:
            c1 = '|'
        if leds[6]:
            c2 = '_'
        if leds[5]:
            c3 = '|'
        ret += '%c%c%c' % (c1, c2, c3)
        return ret

    def _setValue(self, n):
        if n is None:
            self._value = n
        else:
            self._value = str(n)

    def _setLedPenWidth(self, w):
        self._ledPenWidth = w

    def _setLedOnPenColor(self, penColor):
        self._ledOnPenColor = penColor

    def _setLedOnBgColor(self, bgColor):
        self._ledOnBgColor = bgColor

    def _setLedOffPenColor(self, penColor):
        self._ledOffPenColor = penColor

    def _setLedOffBgColor(self, bgColor):
        self._ledOffBgColor = bgColor

    def _setAspectRatio(self, aspectRatio):
        self._aspectRatio = aspectRatio

    def _setBgBrush(self, bgBrush):
        if isinstance(bgBrush, Qt.QColor):
            bgBrush = Qt.QBrush(bgBrush, Qt.Qt.SolidPattern)
        self._bgBrush = bgBrush

    def _setUseFrame(self, useFrame):
        self._useFrame = useFrame

    def getLedOnPenColor(self):
        return self._ledOnPenColor

    def setLedOnPenColor(self, penColor):
        self._setLedOnPenColor(penColor)
        self._updatePensAndBrushes()

    def resetLedOnPenColor(self):
        self.setLenOnPenColor(Q7SegDigit.DftLedOnPenColor)

    def getLedOnBgColor(self):
        return self._ledOnBgColor

    def setLedOnBgColor(self, bgColor):
        self._setLedOnBgColor(bgColor)
        self._updatePensAndBrushes()

    def resetLedOnBgColor(self):
        self.setLedOnBgColor(Q7SegDigit.DftLedOnBgColor)

    def getLedOffPenColor(self):
        return self._ledOffPenColor

    def setLedOffPenColor(self, penColor):
        self._setLedOffPenColor(penColor)
        self._updatePensAndBrushes()

    def resetLedOffPenColor(self):
        self.setLenOffPenColor(Q7SegDigit.DftLedOffPenColor)

    def getLedOffBgColor(self):
        return self._ledOffBgColor

    def setLedOffBgColor(self, bgColor):
        self._setLedOffBgColor(bgColor)
        self._updatePensAndBrushes()

    def resetLedOffBgColor(self):
        self.setLedOffBgColor(Q7SegDigit.DftLedOffBgColor)

    def getBgBrush(self):
        return self._bgBrush

    def setBgBrush(self, bgBrush):
        self._setBgBrush(bgBrush)
        self.update()

    def resetBgBrush(self):
        self.setBgBrush(Q7SegDigit.DftBgBrush)

    def setValue(self, n):
        self._setValue(n)
        self.update()

    def getValue(self):
        return self._value

    def resetValue(self):
        self.setValue(Q7SegDigit.DftValue)

    def setLedPenWidth(self, w):
        self._setLedPenWidth(w)
        self._updatePensAndBrushes()

    def getLedPenWidth(self):
        return self._ledPenWidth

    def resetLedPenWidth(self):
        self.setLedPenWidth(Q7SegDigit.DftLenPenWidth)

    def setAspectRatio(self, apectRatio):
        self._setAspectRatio(apectRatio)
        self.update()

    def getAspectRatio(self):
        return self._aspectRatio

    def resetAspectRatio(self):
        self.setAspectRatio(Q7SegDigit.DftAspectRatio)

    def setUseFrame(self, useFrame):
        self._setUseFrame(useFrame)

    def getUseFrame(self):
        return self._useFrame

    def resetUseFrame(self):
        self.setUseFrame(Q7SegDigit.DftUseFrame)

    #: This property holds the led pen color when the led is light ON
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getLedOnPenColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setLedOnPenColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetLedOnPenColor`
    #:
    ledOnPenColor = Qt.pyqtProperty("QColor", getLedOnPenColor,
                                    setLedOnPenColor, resetLedOnPenColor)

    #: This property holds the led background color when the led is light ON
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getLedOnBgColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setLedOnBgColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetLedOnBgColor`
    #:
    ledOnBgColor = Qt.pyqtProperty("QColor", getLedOnBgColor,
                                   setLedOnBgColor, resetLedOnBgColor)

    #: This property holds the led pen color when the led is light OFF
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getLedOffPenColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setLedOffPenColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetLedOffPenColor`
    #:
    ledOffPenColor = Qt.pyqtProperty("QColor", getLedOffPenColor,
                                     setLedOffPenColor, resetLedOffPenColor)

    #: This property holds the led background color when the led is light OFF
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getLedOffBgColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setLedOffBgColor`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetLedOffBgColor`
    #:
    ledOffBgColor = Qt.pyqtProperty("QColor", getLedOffBgColor,
                                    setLedOffBgColor, resetLedOffBgColor)

    #: This property holds the background brush
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getBgBrush`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setBgBrush`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetBgBrush`
    #:
    bgBrush = Qt.pyqtProperty("QBrush", getBgBrush, setBgBrush, resetBgBrush)

    #: This property holds the pen width
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getLedPenWidth`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setLedPenWidth`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetLedPenWidth`
    #:
    ledPenWidth = Qt.pyqtProperty("int", getLedPenWidth,
                                  setLedPenWidth, resetLedPenWidth)

    #: This property holds wheater of not to draw a frame
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getUseFrame`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setUseFrame`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetUseFrame`
    #:
    useFrame = Qt.pyqtProperty("bool", getUseFrame, setUseFrame, resetUseFrame)

    #: This property holds the widget value
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.getValue`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.setValue`
    #:     * :meth:`taurus.qt.qtgui.display.Q7SegDigit.resetValue`
    #:
    value = Qt.pyqtProperty("QString", getValue, setValue, resetValue)


class Q7SegDisplay(Qt.QWidget):

    def __init__(self, qt_parent=None):
        Qt.QWidget.__init__(self, qt_parent)
        self.__init()

    def __init(self):
        l = Qt.QHBoxLayout()
        l.setSpacing(10)
        l.setMargin(5)
        self.setLayout(l)

        self._digits = []
        for i in xrange(5):
            d = Q7SegDigit()
            d.setUseFrame(False)
            d.setValue(i)
            l.addWidget(d, 0)
            self._digits.append(d)
        self.setAutoFillBackground(True)
        palette = Qt.QPalette()
        brush = Qt.QBrush(Qt.Qt.black)
        palette.setBrush(Qt.QPalette.Active, Qt.QPalette.Window, brush)
        self.setPalette(palette)


def __setBgBrush():
    global digitWidget
    color = Qt.QColorDialog.getColor(Qt.Qt.red)
    if color.isValid():
        digitWidget.setBgBrush(color)


def __setLedOnPenColor():
    global digitWidget
    color = Qt.QColorDialog.getColor(Qt.Qt.red)
    if color.isValid():
        digitWidget.setLedOnPenColor(color)


def __setLedOnBgColor():
    global digitWidget
    color = Qt.QColorDialog.getColor(Qt.Qt.red)
    if color.isValid():
        digitWidget.setLedOnBgColor(color)


def __setLedOffPenColor():
    global digitWidget
    color = Qt.QColorDialog.getColor(Qt.Qt.red)
    if color.isValid():
        digitWidget.setLedOffPenColor(color)


def __setLedOffBgColor():
    global digitWidget
    color = Qt.QColorDialog.getColor(Qt.Qt.red)
    if color.isValid():
        digitWidget.setLedOffBgColor(color)


def main1():
    import sys
    a = Qt.QApplication([])
    panel = Qt.QWidget()
    l = Qt.QHBoxLayout(panel)
    digitWidget = Q7SegDigit()
    digitWidget.setValue(int(sys.argv[1]))
    l.addWidget(digitWidget)
    l.addWidget(Q7SegDigit())
    l.addWidget(Q7SegDigit())
    l.setSpacing(0)
    panel.setVisible(True)
    a.exec_()


def main2():
    import sys
    a = Qt.QApplication([])
    dw = Q7SegDigit()
    dw.setValue(int(sys.argv[1]))
    dw.setVisible(True)
    print dw
    a.exec_()


def main3():
    a = Qt.QApplication([])
    dw = Q7SegDisplay()
    # dw.setValue(int(sys.argv[1]))
    dw.setVisible(True)
    a.exec_()


def main():
    global digitWidget
    a = Qt.QApplication([])
    panel = Qt.QWidget()
    l = Qt.QFormLayout(panel)
    digitWidget = Q7SegDigit()
    digitWidget.setStyleSheet("background-color:red;")
    valueLineEdit = Qt.QLineEdit(digitWidget.getValue())
    valueLineEdit.setMaxLength(1)
    ledPenWidthWidget = Qt.QSpinBox()
    ledPenWidthWidget.setRange(0, 20)
    ledPenWidthWidget.setValue(digitWidget.getLedPenWidth())
    bgBrushWidget = Qt.QPushButton("Bg brush")
    LedOnPenColor = Qt.QPushButton("ON pen color")
    LedOnBgColor = Qt.QPushButton("ON bg color")
    LedOffPenColor = Qt.QPushButton("OFF pen color")
    LedOffBgColor = Qt.QPushButton("OFF bg color")

    l.addRow("Value", digitWidget)
    l.addRow("Digit", valueLineEdit)
    l.addRow("Pen width", ledPenWidthWidget)
    l.addRow(bgBrushWidget)
    l.addRow(LedOnPenColor)
    l.addRow(LedOnBgColor)
    l.addRow(LedOffPenColor)
    l.addRow(LedOffBgColor)
    valueLineEdit.textChanged.connect(digitWidget.setValue)
    ledPenWidthWidget.valueChanged.connect(digitWidget.setLedPenWidth)
    bgBrushWidget.clicked.connect(__setBgBrush)
    LedOnPenColor.clicked.connect(__setLedOnPenColor)
    LedOnBgColor.clicked.connect(__setLedOnBgColor)
    LedOffPenColor.clicked.connect(__setLedOffPenColor)
    LedOffBgColor.clicked.connect(__setLedOffBgColor)
    panel.setVisible(True)
    a.exec_()


if __name__ == "__main__":
    main3()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Unit tests for taurusbase"""


from taurus.external import unittest
from taurus.test import insertTest
from taurus.qt.qtgui.test import BaseWidgetTestCase
from taurus.core.tango.test import TangoSchemeTestLauncher
from taurus.qt.qtgui.container import TaurusWidget

DEV_NAME = TangoSchemeTestLauncher.DEV_NAME


@insertTest(helper_name='getDisplayValue',
            model='eval:1+2#',
            expected='-----')
@insertTest(helper_name='getDisplayValue',
            model='eval:1+2#label',
            expected='1+2')
@insertTest(helper_name='getDisplayValue',
            model='eval:1+2',
            expected='3 ')  # @TODO: change to '3' if/when pint supports it
# This checks if the pre-tep3 behavior is kept (and it fails)
# ...but I think it should *not* be kept
@insertTest(helper_name='getDisplayValue',
            model='tango://' + DEV_NAME + '/double_scalar?configuration',
            expected='double_scalar?configuration',
            test_skip="old behaviour which we probably don't want")
@insertTest(helper_name='getDisplayValue',
            model='tango://' + DEV_NAME + '/float_scalar?configuration=label',
            expected='float_scalar')
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/double_scalar#rvalue.magnitude',
            expected='1.23')
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/float_scalar#label',
            expected='float_scalar')
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/float_scalar#',
            expected='-----')
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/state',
            expected='ON')
# This fails due to encode/decode rounding errors for float<-->numpy.float32
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/float_scalar',
            expected='1.23 mm',
            test_skip='enc/decode rounding errors for float<-->numpy.float32')
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/double_scalar',
            expected='1.23 mm')
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/short_scalar',
            expected='123 mm')
@insertTest(helper_name='getDisplayValue',
            model='tango:' + DEV_NAME + '/boolean_scalar',
            expected='True')
class GetDisplayValueTestCase(TangoSchemeTestLauncher, BaseWidgetTestCase,
                              unittest.TestCase):
    """Check TaurusBaseComponent.getDisplayValue
    """
    _klass = TaurusWidget

    def setUp(self):
        BaseWidgetTestCase.setUp(self)

    # def tearDown(self):
    #     BaseWidgetTestCase.tearDown(self)

    def getDisplayValue(self, model=None, expected=None):
        '''Check if setModel works when using parent model'''
        self._widget.setModel(model)
        got = self._widget.getDisplayValue()
        msg = ('getDisplayValue for "%s" should be %r (got %r)' %
               (model, expected, got))
        self.assertEqual(expected, got, msg)
        self.assertMaxDeprecations(0)



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""adapted from http://code.activestate.com/recipes/576576/"""

__all__ = ["ThreadPool", "Worker"]

__docformat__ = "restructuredtext"

from threading import Thread, currentThread
from Queue import Queue
from time import sleep, time
from traceback import extract_stack, format_list

from prop import propertx
from log import Logger, DebugIt, TraceIt


class ThreadPool(Logger):
    """"""

    NoJob = 6 * (None,)

    def __init__(self, name=None, parent=None, Psize=20, Qsize=20, daemons=True):
        Logger.__init__(self, name, parent)
        self._daemons = daemons
        self.localThreadId = 0
        self.workers = []
        self.jobs = Queue(Qsize)
        self.size = Psize
        self.accept = True

    @propertx
    def size():
        def set(self, newSize):
            """set method for the size property"""
            nb_workers = len(self.workers)
            if newSize == nb_workers:
                return

            for i in range(newSize - nb_workers):
                self.localThreadId += 1
                name = "%s.W%03i" % (self.log_name, self.localThreadId)
                new = Worker(self, name, self._daemons)
                self.workers.append(new)
                self.debug("Starting %s" % name)
                new.start()

            # remove the old worker threads
            nb_workers = len(self.workers)
            for i in range(nb_workers - newSize):
                self.jobs.put(self.NoJob)

        def get(self):
            """get method for the size property"""
            return len(self.workers)

        return get, set, None, "number of threads"

    def add(self, job, callback=None, *args, **kw):
        if self.accept:
            # first gather some information on the object which requested the
            # job in case the job throws an exception
            th_id, stack = currentThread().name, extract_stack()[:-1]
            self.jobs.put((job, args, kw, callback, th_id, stack))

    def join(self):
        self.accept = False
        while True:
            for w in self.workers:
                if w.isAlive():
                    self.jobs.put(self.NoJob)
                    break
            else:
                break

    @property
    def qsize(self):
        return self.jobs.qsize()

    def getNumOfBusyWorkers(self):
        ''' Get the number of workers that are in busy mode.
        '''
        n = 0
        for w in self.workers:
            if w.isBusy():
                n += 1
        return n


class Worker(Thread, Logger):

    def __init__(self, pool, name=None, daemon=True):
        name = name or self.__class__.__name__
        Thread.__init__(self, name=name)
        Logger.__init__(self, name, pool)
        self.daemon = daemon
        self.pool = pool
        self.cmd = ''
        self.busy = False

    def run(self):
        get = self.pool.jobs.get
        while True:
            cmd, args, kw, callback, th_id, stack = get()
            if cmd:
                self.busy = True
                self.cmd = cmd.__name__
                try:
                    if callback:
                        callback(cmd(*args, **kw))
                    else:
                        cmd(*args, **kw)
                except:
                    orig_stack = "".join(format_list(stack))
                    self.error("Uncaught exception running job '%s' called "
                               "from thread %s:\n%s",
                               self.cmd, th_id, orig_stack, exc_info=1)
                finally:
                    self.busy = False
                    self.cmd = ''
            else:
                self.pool.workers.remove(self)
                return

    def isBusy(self):
        return self.busy

if __name__ == '__main__':

    def easyJob(*arg, **kw):
        n = arg[0]
        print '\tSleep\t\t', n
        sleep(n)
        return 'Slept\t%d' % n

    def longJob(*arg, **kw):
        print "\tStart\t\t\t", arg, kw
        n = arg[0] * 3
        sleep(n)
        return "Job done in %d" % n

    def badJob(*a, **k):
        print '\n !!! OOOPS !!!\n'
        a = 1 / 0

    def show(*arg, **kw):
        print 'callback : %s' % arg[0]

    def test_1(**kwargs):
        workers = kwargs.pop('workers', 5)
        jobqueue = kwargs.pop('jobqueue', 10)
        pool = ThreadPool(name='ThreadPool', Psize=workers, Qsize=jobqueue)
        print "\n\t\t... let's add some jobs ...\n"
        for j in range(5):
            if j == 1:
                pool.add(badJob)
            for i in range(5, 0, -1):
                pool.add(longJob, show, i)
                pool.add(easyJob, show, i)
        print '''
            \t\t... and now, we're waiting for the %i workers to get the %i jobs done ...
        ''' % (pool.size, pool.qsize)
        sleep(15)
        print "\n\t\t... ok, that may take a while, let's get some reinforcement ...\n"
        sleep(5)
        pool.size = 50
        print '\n\t\t... Joining ...\n'
        pool.join()
        print '\n\t\t... Ok ...\n'

    def test_2(**kwargs):
        workers = kwargs.pop('workers', 5)
        jobqueue = kwargs.pop('jobqueue', 10)
        numjobs = kwargs.pop('numjobs', 10)
        sleep_t = kwargs.pop('sleep_t', 1)
        #from taurus.core.util.threadpool import ThreadPool
        pool = ThreadPool(name='ThreadPool', Psize=workers, Qsize=jobqueue)
        print "\n\t\t... Check the number of busy workers ...\n"
        print "Num of busy workers = %s" % (pool.getNumOfBusyWorkers())
        print "\n\t\t... let's add some jobs ...\n"
        for i in range(numjobs):
            pool.add(easyJob, None, sleep_t)
        print '\n\t\t... Monitoring the busy workers ...\n'
        t0 = time()
        while pool.getNumOfBusyWorkers() > 0:
            print "busy workers = %s" % (pool.getNumOfBusyWorkers())
            sleep(0.5)
        t1 = time()
        print "Run %s jobs of 1 second took %.3f" % (numjobs, t1 - t0)
        print '\n\t\t... Joining ...\n'
        pool.join()
        print '\n\t\t... Ok ...\n'

    def main(argv):
        kwargs = {}
        for arg in argv:
            k, v = arg.split('=')
            kwargs[k] = int(v)
        # Run test
        test_1(**kwargs)
        test_2(**kwargs)

    import sys
    main(sys.argv[1:])



import sys
from taurus.external.qt import Qt
from taurus.qt.qtgui.display import TaurusLabel

app = Qt.QApplication(sys.argv)
panel = Qt.QWidget()
w = TaurusLabel(panel)
w.setModel('sys/taurustest/1/state')
panel.show()

sys.exit(app.exec_())



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module contains the taurus base manager class"""

__all__ = ["TaurusManager"]

__docformat__ = "restructuredtext"

import os
import atexit

from .util.singleton import Singleton
from .util.log import Logger, taurus4_deprecation
from .util.threadpool import ThreadPool

from .taurusbasetypes import OperationMode, ManagerState, TaurusSerializationMode
from .taurusauthority import TaurusAuthority
from .taurusdevice import TaurusDevice
from .taurusattribute import TaurusAttribute
from .taurusexception import TaurusException
from .taurusfactory import TaurusFactory
from .taurushelper import getSchemeFromName
from taurus import tauruscustomsettings


class TaurusManager(Singleton, Logger):
    """A :class:`taurus.core.util.singleton.Singleton` class designed to provide Taurus management.

       Example::

           >>> import taurus.core.taurusmanager
           >>> manager = taurus.core.taurusmanager.TaurusManager()
           >>> print manager == taurus.core.taurusmanager.TaurusManager()
           True
    """
    PLUGIN_KEY = "__taurus_plugin__"

    DefaultSerializationMode = TaurusSerializationMode.Concurrent
    default_scheme = getattr(tauruscustomsettings, 'DEFAULT_SCHEME', "tango")

    def __init__(self):
        """ Initialization. Nothing to be done here for now."""
        pass

    def init(self, *args, **kwargs):
        """Singleton instance initialization.
           For internal usage only. Do **NOT** call this method directly"""
        self._state = ManagerState.UNINITIALIZED
        self.call__init__(Logger)
        self.reInit()
        atexit.register(self.cleanUp)

    def reInit(self):
        """Reinitialization"""
        if self._state == ManagerState.INITED:
            return

        self.trace("reInit()")
        this_path = os.path.abspath(__file__)
        self._this_path = os.path.dirname(this_path)
        self._serialization_mode = self.DefaultSerializationMode
        if self._serialization_mode == TaurusSerializationMode.Concurrent:
            self._thread_pool = ThreadPool(name="TaurusTP",
                                           parent=self,
                                           Psize=5,
                                           Qsize=1000)
        else:
            self._thread_pool = None
        self._plugins = None

        self._initial_default_scheme = self.default_scheme

        self._state = ManagerState.INITED

    def cleanUp(self):
        """Cleanup"""
        if self._state == ManagerState.CLEANED:
            return
        self.trace("cleanUp()")

        if self._plugins is None:
            return
        self.trace("[TaurusManager] cleanUp")
        self._plugins = None

        self._thread_pool.join()
        self._thread_pool = None

        self._state = ManagerState.CLEANED

    def addJob(self, job, callback=None, *args, **kw):
        """Add a new job (callable) to the queue. The new job will be processed
        by a separate thread

        :param job: (callable) a callable object
        :param callback: (callable) called after the job has been processed
        :param args: (list) list of arguments passed to the job
        :param kw: (dict) keyword arguments passed to the job
        """
        if self._serialization_mode == TaurusSerializationMode.Concurrent:
            if not hasattr(self, "_thread_pool") or self._thread_pool is None:
                self.info("Job cannot be processed.")
                self.debug(
                    "The requested job cannot be processed. Make sure this manager is initialized")
                return
            self._thread_pool.add(job, callback, *args, **kw)
        else:
            job(*args, **kw)

    def setSerializationMode(self, mode):
        """Sets the serialization mode for the system.

        :param mode: (TaurusSerializationMode) the new serialization mode"""
        self._serialization_mode = mode

    def getSerializationMode(self):
        """Gives the serialization operation mode.

        :return: (TaurusSerializationMode) the current serialization mode"""
        return self._serialization_mode

    def setOperationMode(self, mode):
        """Deprecated. Sets the operation mode for the system.

        :param mode: (OperationMode) the new operation mode"""
        dep = 'setOperationMode'
        rel = 'Taurus4'
        dbg_msg = "Don't use this method"
        msg = '%s is deprecated (from %s). %s' % (dep, rel, dbg_msg)
        self.deprecated(msg)

    def getOperationMode(self):
        """Deprecated. Gives the current operation mode.

        :return: (OperationMode) the current operation mode"""
        dep = 'getOperationMode'
        rel = 'Taurus4'
        dbg_msg = "Don't use this method"
        msg = '%s is deprecated (from %s). %s' % (dep, rel, dbg_msg)
        self.deprecated(msg)
        return OperationMode.ONLINE

    def getDefaultFactory(self):
        """Gives the default factory.

        :return: (taurus.core.taurusfactory.TaurusFactory) the default taurus factory
        """
        return self.getPlugins().get(self.default_scheme, None)

    def getPlugins(self):
        """Gives the information about the existing plugins

        :return: (dict<str, class taurus.core.taurusfactory.TaurusFactory>)the list of plugins
        """
        if self._plugins is None:
            self._plugins = self._build_plugins()
        return self._plugins

    def getFactory(self, scheme=None):
        """Gives the factory class object supporting the given scheme

        :param scheme: (str or None) the scheme. If None the default scheme is used
        :return: (taurus.core.taurusfactory.TaurusFactory or None) the factory class object for the
                 given scheme or None if a proper factory is not found
        """
        if scheme is None:
            return self.getDefaultFactory()
        return self.getPlugins().get(scheme)

    def getObject(self, cls, name):
        """Gives the object for the given class with the given name

        :param cls: (taurus.core.taurusmodel.TaurusModel) object class
        :param name: (str) the object name
        :return: (taurus.core.taurusmodel.TaurusModel or None) a taurus model object
        """
        factory = self._get_factory(name)
        if factory is None:
            return
        return factory.getObject(cls, name)

    def findObject(self, absolute_name):
        """Finds the object with the given name

        :param absolute_name: (str) the object name
        :return: (taurus.core.taurusmodel.TaurusModel or None) the taurus model object or None if
                 no suitable name found
        """
        factory = self._get_factory(absolute_name)
        if factory is None:
            return
        return factory.findObject(absolute_name)

    def findObjectClass(self, absolute_name):
        """Finds the object class for the given object name

        :param absolute_name: (str) the object name
        :return: (class taurus.core.taurusmodel.TaurusModel or None) the taurus model class object or
                 None if no suitable name found
        """
        factory = self._get_factory(absolute_name)
        if factory is None:
            return
        return factory.findObjectClass(absolute_name)

    def getAuthority(self, name):
        """Returns a database object for the given name

        :param name: (str) database name
        :return: (taurus.core.taurusauthority.TaurusAuthority) the authority for the given name
        """
        return self.getObject(TaurusAuthority, name)

    def getDatabase(self, name):
        """Deprecated. Use getAuthority instead"""
        self.warning('getDatabase is deprecated. Use getAuthority instead')
        return self.getAuthority(self, name)

    def getDevice(self, name):
        """Returns a device object for the given name

        :param name: (str) device name
        :return: (taurus.core.taurusdevice.TaurusDevice) the device for the given name
        """
        return self.getObject(TaurusDevice, name)

    def getAttribute(self, name):
        """Returns a attribute object for the given name

        :param name: (str) attribute name
        :return: (taurus.core.taurusattribute.TaurusAttribute) the attribute for the given name
        """
        return self.getObject(TaurusAttribute, name)

    @taurus4_deprecation(alt='getAttribute')
    def getConfiguration(self, name):
        """Returns a configuration object for the given name

        :param name: (str) configuration name
        :return: (taurus.core.taurusconfiguration.TaurusConfiguration) the configuration for the given name
        """
        return self.getAttribute(name)

    def _get_factory(self, name):
        scheme = self.getScheme(name)
        if scheme is None:
            return
        try:
            return self.getPlugins()[scheme]()
        except:
            raise TaurusException('Invalid scheme "%s"' % scheme)

    def getScheme(self, name):
        '''Returns the scheme name for a given model name

        :param name: (str) model name
        :return: (str) scheme name
        '''
        return getSchemeFromName(name, implicit=True)

    def _get_schema(self, name):
        raise DeprecationWarning(
            '_get_schema is deprecated. Use getScheme instead')

    def _build_plugins(self):
        plugin_classes = self._get_plugin_classes()
        plugins = {}
        for plugin_class in plugin_classes:
            schemes = list(plugin_class.schemes)
            for scheme in schemes:
                if plugins.has_key(scheme):
                    k = plugins[scheme]
                    self.warning("Conflicting plugins: %s and %s both implement "
                                 "scheme %s. Will keep using %s" % (k.__name__,
                                                                    plugin_class.__name__, scheme, k.__name__))
                else:
                    plugins[scheme] = plugin_class
        return plugins

    def buildPlugins(self):
        '''Returns the current valid plugins

        :return: (dic) plugins
        '''
        return self._build_plugins()

    def _get_plugin_classes(self):
        upgrade_classes = []

        elems = os.listdir(self._this_path)
        dirs = []
        for elem in elems:
            if elem.startswith('.') or elem.startswith("_"):
                continue
            elem = os.path.join(self._this_path, elem)
            if not os.path.isdir(elem):
                continue
            plugin_file = os.path.join(elem, self.PLUGIN_KEY)
            if not os.path.exists(plugin_file):
                continue
            if not os.path.exists(os.path.join(elem, '__init__.py')):
                continue
            dirs.append(elem)

        plugins = []

        full_module_names = ['taurus.core.%s' %
                             d.split(os.path.sep)[-1] for d in dirs]
        from taurus import tauruscustomsettings
        full_module_names.extend(
            getattr(tauruscustomsettings, 'EXTRA_SCHEME_MODULES', []))

        for full_module_name in full_module_names:
            try:
                m = __import__(full_module_name, fromlist=['*'], level=0)
            except Exception, imp1:
                # just in case we are in python 2.4
                try:
                    m = __import__(full_module_name,
                                   globals(), locals(), ['*'])
                except:
                    self.debug('Failed to inspect %s' % (full_module_name))
                    self.debug('Details:', exc_info=1)
                    continue
            for s in m.__dict__.values():
                plugin = None
                try:
                    if issubclass(s, TaurusFactory) and \
                       issubclass(s, Singleton):
                        if hasattr(s, 'schemes'):
                            schemes = getattr(s, 'schemes')
                            if len(schemes):
                                plugin = s
                            else:
                                scheme = self._find_scheme(s)
                                if not scheme is None:
                                    s.schemes = (scheme,)
                                    plugin = s
                except:
                    pass
                if not plugin is None:
                    self.debug('Found plugin %s' % plugin.__name__)
                    plugins.append(plugin)
        return plugins

    def _find_scheme(self, factory_class):
        class_name = factory_class.__name__
        for i in xrange(1, len(class_name)):
            if class_name[i].isupper():
                return class_name[:i].lower()

    def applyPendingOperations(self, ops):
        """Executes the given operations

        :param ops: the sequence of operations
        :type ops: sequence<taurus.core.taurusoperation.TaurusOperation>"""
        for o in ops:
            o.execute()

    def changeDefaultPollingPeriod(self, period):
        plugin_classes = self._get_plugin_classes()
        for plugin_class in plugin_classes:
            scheme = plugin_class.schemes[0]
            self.getFactory(scheme)().changeDefaultPollingPeriod(period)

    def __str__name__(self, name):
        return '{0}({1})'.format(self.__class__.__name__, name)

    def __str__(self):
        return self.__str__name__("")

    def __repr__(self):
        return self.__str__name__("")

if __name__ == '__main__':
    manager = TaurusManager()
    print manager.getPlugins()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a set of basic Taurus widgets based on QLed"""

__all__ = ["TaurusLed"]

__docformat__ = 'restructuredtext'

import weakref
import operator

from taurus.external.qt import Qt

from taurus.core import DataFormat, AttrQuality
from taurus.core.tango import DevState

from taurus.qt.qtgui.base import TaurusBaseWidget
from qled import QLed

_QT_PLUGIN_INFO = {
    'module': 'taurus.qt.qtgui.display',
    'group': 'Taurus Display',
    'icon': "designer:ledgreen.png",
}


class _TaurusLedController(object):

    #            key     status,     color, inTrouble
    LedMap = {True: (True,    "green",    False),
              False: (False,    "black",    False),
              None: (False,    "black",     True)}

    LedQualityMap = {
        AttrQuality.ATTR_ALARM: (True,   "orange",    False),
        AttrQuality.ATTR_CHANGING: (True,     "blue",    False),
        AttrQuality.ATTR_INVALID: (True,      "red",    False),
        AttrQuality.ATTR_VALID: (True,    "green",    False),
        AttrQuality.ATTR_WARNING: (True,   "orange",    False),
        None: (False,    "black",     True)}

    def __init__(self, widget):
        self._widget = weakref.ref(widget)

    def widget(self):
        return self._widget()

    def modelObj(self):
        return self.widget().getModelObj()

    def value(self):
        widget, obj = self.widget(), self.modelObj()
        fgRole = widget.fgRole
        value = None
        if fgRole == 'rvalue':
            value = obj.rvalue
        elif fgRole == 'wvalue':
            value = obj.wvalue
        elif fgRole == 'quality':
            return obj.quality

        # handle 1D and 2D values
        if obj.data_format == DataFormat._0D:
            return value

        idx = widget.getModelIndexValue()
        if idx is None or len(idx) == 0:
            return value

        for i in idx:
            value = value[i]
        return value

    def usePreferedColor(self, widget):
        return True

    def handleEvent(self, evt_src, evt_type, evt_value):
        self.update()

    def update(self):
        widget = self.widget()

        self._updateDisplay(widget)
        self._updateToolTip(widget)

    def _updateDisplay(self, widget):
        key = None
        try:
            key = self.value()
        except Exception:
            pass
        ledMap = self.LedMap
        if widget.fgRole == 'quality':
            ledMap = self.LedQualityMap
        try:
            status, color, trouble = ledMap[key]
        except:
            status, color, trouble = False, "red", True
        if self.usePreferedColor(widget):
            if status:
                color = widget.onColor
            else:
                color = widget.offColor
        widget.ledStatus = status
        widget.ledColor = color
        if trouble:
            widget.setAutoFillBackground(True)
            bg_brush = Qt.QBrush(Qt.Qt.BDiagPattern)
            palette = widget.palette()
            palette.setBrush(Qt.QPalette.Window, bg_brush)
            palette.setBrush(Qt.QPalette.Base, bg_brush)
            widget.setPalette(palette)
        else:
            widget.setAutoFillBackground(False)

    def _updateToolTip(self, widget):
        widget.setToolTip(widget.getFormatedToolTip())


class _TaurusLedControllerBool(_TaurusLedController):

    def usePreferedColor(self, widget):
        # use prefered widget color if representing the boolean read or write
        # value. If representing the quality, use the quality map
        return widget.fgRole != 'quality'


class _TaurusLedControllerState(_TaurusLedController):

    #                key      status,       color, inTrouble
    LedMap = {DevState.ON: (True,    "green",    False),
              DevState.OFF: (False,    "black",    False),
              DevState.CLOSE: (True,    "white",    False),
              DevState.OPEN: (True,    "green",    False),
              DevState.INSERT: (True,    "green",    False),
              DevState.EXTRACT: (True,    "green",    False),
              DevState.MOVING: (True,     "blue",    False),
              DevState.STANDBY: (True,   "yellow",    False),
              DevState.FAULT: (True,      "red",    False),
              DevState.INIT: (True,   "yellow",    False),
              DevState.RUNNING: (True,     "blue",    False),
              DevState.ALARM: (True,   "orange",    False),
              DevState.DISABLE: (True,  "magenta",    False),
              DevState.UNKNOWN: (False,    "black",    False),
              None: (False,    "black",     True)}

    def usePreferedColor(self, widget):
        # never use prefered widget color. Use always the map
        return False


class _TaurusLedControllerDesignMode(_TaurusLedController):

    def _updateDisplay(self, widget):
        widget.ledStatus = True
        if widget.ledStatus:
            widget.ledColor = widget.onColor
        else:
            widget.ledColor = widget.offColor
        widget.setAutoFillBackground(False)

    def _updateToolTip(self, widget):
        widget.setToolTip("Design mode TaurusLed")


class TaurusLed(QLed, TaurusBaseWidget):
    """A widget designed to represent with a LED image the state of a device,
    the value of a boolean attribute or the quality of an attribute."""

    DefaultModelIndex = None
    DefaultFgRole = 'rvalue'
    DefaultOnColor = "green"
    DefaultOffColor = "black"

    _deprecatedRoles = dict(value='rvalue', w_value='wvalue')

    def __init__(self, parent=None, designMode=False):
        name = self.__class__.__name__
        self._designMode = designMode
        self._modelIndex = self.DefaultModelIndex
        self._modelIndexStr = ''
        self._fgRole = self.DefaultFgRole
        self._onColor = self.DefaultOnColor
        self._offColor = self.DefaultOffColor
        self._controller = None
        self.call__init__wo_kw(QLed, parent)
        self.call__init__(TaurusBaseWidget, name, designMode=designMode)

        # if we are in design mode there will be no events so we force the
        # creation of a controller object
        if self._designMode:
            self.controller().update()

    def _calculate_controller_class(self):
        model = self.getModelObj()

        klass = _TaurusLedController
        if self._designMode:
            klass = _TaurusLedControllerDesignMode
        elif model is None:
            klass = _TaurusLedController
        elif model.isBoolean():
            klass = _TaurusLedControllerBool
        elif model.isState():
            klass = _TaurusLedControllerState
        return klass

    def controller(self):
        ctrl = self._controller
        # if there is a controller object and it is not the base controller...
        if ctrl is not None and not ctrl.__class__ == _TaurusLedController:
            return ctrl

        # if there is a controller object and it is still the same class...
        ctrl_klass = self._calculate_controller_class()
        if ctrl is not None and ctrl.__class__ == ctrl_klass:
            return ctrl

        self._controller = ctrl = ctrl_klass(self)
        return ctrl

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TaurusBaseWidget overwriting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def handleEvent(self, evt_src, evt_type, evt_value):
        self.controller().handleEvent(evt_src, evt_type, evt_value)

    def isReadOnly(self):
        return True

    def setModel(self, m):
        # force to build another controller
        self._controller = None
        TaurusBaseWidget.setModel(self, m)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT property definition
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getFgRole(self):
        return self._fgRole

    def setFgRole(self, fgRole):
        role = self._deprecatedRoles.get(fgRole, fgRole)
        if fgRole != role:
            self.deprecated(rel='4.0', dep='setFgRole(%s)' % fgRole,
                            alt='setFgRole(%s)' % role)
        self._fgRole = str(role)
        self.controller().update()

    def resetFgRole(self):
        self.setFgRole(self.DefaultFgRole)

    def getOnColor(self):
        """Returns the preferred led on color
        :return: led on color
        :rtype: str"""
        return self._onColor

    def setOnColor(self, color):
        """Sets the preferred led on color
        :param status: the new on color
        :type  status: str"""
        color = str(color).lower()
        if not self.isLedColorValid(color):
            raise Exception("Invalid color '%s'" % color)
        self._onColor = color
        self.controller().update()

    def resetOnColor(self):
        """Resets the preferred led on color"""
        self.setOnColor(self.DefaultOnColor)

    def getOffColor(self):
        """Returns the preferred led off color
        :return: led off color
        :rtype: str"""
        return self._offColor

    def setOffColor(self, color):
        """Sets the preferred led off color
        :param status: the new off color
        :type  status: str"""
        color = str(color).lower()
        if not self.isLedColorValid(color):
            raise Exception("Invalid color '%s'" % color)
        self._offColor = color
        self.controller().update()

    def resetOffColor(self):
        """Resets the preferred led color"""
        self.setOffColor(self.DefaultOffColor)

    def getModelIndexValue(self):
        return self._modelIndex

    def getModelIndex(self):
        return self._modelIndexStr

    def setModelIndex(self, modelIndex):
        mi = str(modelIndex)
        if len(mi) == 0:
            self._modelIndex = None
        else:
            try:
                mi_value = eval(str(mi))
            except:
                return
            if type(mi_value) == int:
                mi_value = mi_value,
            if not operator.isSequenceType(mi_value):
                return
            self._modelIndex = mi_value
        self._modelIndexStr = mi
        self.controller().update()

    def resetModelIndex(self):
        self.setModelIndex(self.DefaultModelIndex)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        d = TaurusBaseWidget.getQtDesignerPluginInfo()
        d.update(_QT_PLUGIN_INFO)
        return d

    #: This property holds the unique URI string representing the model name
    #: with which this widget will get its data from. The convention used for
    #: the string can be found :ref:`here <model-concept>`.
    #:
    #: In case the property :attr:`useParentModel` is set to True, the model
    #: text must start with a '/' followed by the attribute name.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusBaseWidget.getModel`
    #:     * :meth:`TaurusLabel.setModel`
    #:     * :meth:`TaurusBaseWidget.resetModel`
    #:
    #: .. seealso:: :ref:`model-concept`
    model = Qt.pyqtProperty("QString", TaurusBaseWidget.getModel, setModel,
                            TaurusBaseWidget.resetModel)

    #: This property holds whether or not this widget should search in the
    #: widget hierarchy for a model prefix in a parent widget.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusBaseWidget.getUseParentModel`
    #:     * :meth:`TaurusBaseWidget.setUseParentModel`
    #:     * :meth:`TaurusBaseWidget.resetUseParentModel`
    #:
    #: .. seealso:: :ref:`model-concept`
    useParentModel = Qt.pyqtProperty("bool", TaurusBaseWidget.getUseParentModel,
                                     TaurusBaseWidget.setUseParentModel,
                                     TaurusBaseWidget.resetUseParentModel)

    #: This property holds the index inside the model value that should be
    #: displayed
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLed.getModelIndex`
    #:     * :meth:`TaurusLed.setModelIndex`
    #:     * :meth:`TaurusLed.resetModelIndex`
    #:
    #: .. seealso:: :ref:`model-concept`
    modelIndex = Qt.pyqtProperty("QString", getModelIndex, setModelIndex,
                                 resetModelIndex)

    #: This property holds the foreground role.
    #: Valid values are:
    #:
    #:     #. 'value' - the value is used
    #:     #. 'w_value' - the write value is used
    #:     #. 'quality' - the quality is used
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLed.getFgRole`
    #:     * :meth:`TaurusLed.setFgRole`
    #:     * :meth:`TaurusLed.resetFgRole`
    fgRole = Qt.pyqtProperty("QString", getFgRole, setFgRole,
                             resetFgRole, doc="foreground role")

    #: This property holds the preferred led color
    #: This value is used for the cases where the model value does not contain
    #: enough information to distinguish between different On colors.
    #: For example, a bool attribute, when it is False it is displayed with the
    #: off led but when it is true it may be displayed On in any color. The
    #: prefered color would be used in this case.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLed.getOnColor`
    #:     * :meth:`TaurusLed.setOnColor`
    #:     * :meth:`TaurusLed.resetOnColor`
    onColor = Qt.pyqtProperty("QString", getOnColor, setOnColor, resetOnColor,
                              doc="preferred led On color")

    #: This property holds the preferred led color
    #: This value is used for the cases where the model value does not contain
    #: enough information to distinguish between different Off colors.
    #: For example, a bool attribute, when it is False it is displayed with the
    #: off led but when it is true it may be displayed On in any color. The
    #: prefered color would be used in this case.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLed.getOffColor`
    #:     * :meth:`TaurusLed.setOffColor`
    #:     * :meth:`TaurusLed.resetOffColor`
    offColor = Qt.pyqtProperty("QString", getOffColor, setOffColor, resetOffColor,
                               doc="preferred led Off color")


def demo():
    "Led"
    import demo.taurusleddemo
    return demo.taurusleddemo.main()


def main():
    import sys
    import taurus.qt.qtgui.application
    Application = taurus.qt.qtgui.application.TaurusApplication

    app = Application.instance()
    owns_app = app is None

    if owns_app:
        import taurus.core.util.argparse
        parser = taurus.core.util.argparse.get_taurus_parser()
        parser.usage = "%prog [options] <full_attribute_name(s)>"
        app = Application(sys.argv, cmd_line_parser=parser,
                          app_name="Taurus led demo", app_version="1.0",
                          org_domain="Taurus", org_name="Tango community")

    args = app.get_command_line_args()

    if len(args) == 0:
        w = demo()
    else:
        models = map(str.lower, args)

        w = Qt.QWidget()
        layout = Qt.QGridLayout()
        w.setLayout(layout)
        for model in models:
            led = TaurusLed()
            led.model = model
            layout.addWidget(led)
    w.show()

    if owns_app:
        sys.exit(app.exec_())
    else:
        return w

if __name__ == '__main__':
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a set of basic taurus widgets based on QAbstractSpinBox"""

__all__ = ["TaurusValueSpinBox", "TaurusValueSpinBoxEx"]

__docformat__ = 'restructuredtext'

from taurus.external.qt import Qt

from taurus.qt.qtgui.base import TaurusBaseWritableWidget
from tauruslineedit import TaurusValueLineEdit
from taurus.qt.qtgui.icon import getStandardIcon


class TaurusValueSpinBox(Qt.QAbstractSpinBox):

    modelChanged = Qt.pyqtSignal('const QString &')

    def __init__(self, qt_parent=None, designMode=False):
        Qt.QAbstractSpinBox.__init__(self, qt_parent)

        # Overwrite not to show quality by default
        self._showQuality = False

        self._singleStep = 1.0

        lineEdit = TaurusValueLineEdit(designMode=designMode)
        self.setLineEdit(lineEdit)
        self.setAccelerated(True)
        self.editingFinished.connect(self.writeValue)

    def __getattr__(self, name):
        return getattr(self.lineEdit(), name)

    def setValue(self, v):
        self.lineEdit().setValue(v)

    def getValue(self):
        return self.lineEdit().getValue()

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Mandatory overload from QAbstractSpinBox
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def stepBy(self, steps):
        #validator = self.lineEdit().validator()
        self.setValue(self.getValue() + self.getSingleStep() * steps)

        if self.lineEdit().getAutoApply():
            self.lineEdit().editingFinished.emit()
        else:
            kmods = Qt.QCoreApplication.instance().keyboardModifiers()
            controlpressed = bool(kmods & Qt.Qt.ControlModifier)
            if controlpressed:
                self.lineEdit().writeValue(forceApply=True)

    def stepEnabled(self):
        ret = Qt.QAbstractSpinBox.StepEnabled(Qt.QAbstractSpinBox.StepNone)
        if self.getModelObj() is None:
            return ret
        if self.getModelObj().getValueObj() is None:
            return ret

        le, curr_val, ss = self.lineEdit(), self.getValue(), self.getSingleStep()

        if curr_val is None:
            return ret

        if not le._outOfRange(curr_val + ss):
            ret |= Qt.QAbstractSpinBox.StepUpEnabled
        if not le._outOfRange(curr_val - ss):
            ret |= Qt.QAbstractSpinBox.StepDownEnabled
        return ret

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # Model related methods
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def setModel(self, model):
        self.lineEdit().setModel(model)

    def getModel(self):
        return self.lineEdit().getModel()

    def resetModel(self):
        return self.lineEdit().resetModel()

    def setUseParentModel(self, model):
        self.lineEdit().setUseParentModel(model)

    def getUseParentModel(self):
        return self.lineEdit().getUseParentModel()

    def resetUseParentModel(self):
        return self.lineEdit().resetUseParentModel()

    def setAutoApply(self, model):
        self.lineEdit().setAutoApply(model)

    def getAutoApply(self):
        return self.lineEdit().getAutoApply()

    def resetAutoApply(self):
        return self.lineEdit().resetAutoApply()

    def setForcedApply(self, model):
        self.lineEdit().setForcedApply(model)

    def getForcedApply(self):
        return self.lineEdit().getForcedApply()

    def resetForcedApply(self):
        return self.lineEdit().resetForcedApply()

    def getSingleStep(self):
        return self._singleStep

    @Qt.pyqtSlot(float)
    def setSingleStep(self, step):
        self._singleStep = step

    def resetSingleStep(self):
        self.setSingleStep(1.0)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return {
            'group': 'Taurus Input',
            'icon': 'designer:spinbox.png',
            'module': 'taurus.qt.qtgui.input',
            'container': False}

    singleStep = Qt.pyqtProperty("double", getSingleStep, setSingleStep,
                                 resetSingleStep)

    model = Qt.pyqtProperty("QString", getModel, setModel, resetModel)

    useParentModel = Qt.pyqtProperty("bool", getUseParentModel,
                                     setUseParentModel, resetUseParentModel)

    autoApply = Qt.pyqtProperty("bool", getAutoApply, setAutoApply,
                                resetAutoApply)

    forcedApply = Qt.pyqtProperty("bool", getForcedApply, setForcedApply,
                                  resetForcedApply)

_S = """
QSpinBox::up-button {
    border-width: 0px;
}
QPushButton {
    min-width: 6px;
    min-height: 10px;
    /*border-style: solid;
    border-width: 1px;
    border-color: black;
    border-top-right-radius: 2px;
    border-bottom-right-radius: 2px;*/
    margin: 0px;
    padding: 0px;
}
"""


class TaurusValueSpinBoxEx(Qt.QWidget):

    def __init__(self, qt_parent=None, designMode=False):
        Qt.QWidget.__init__(self, qt_parent)
        layout = Qt.QGridLayout()
        layout.setMargin(0)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        self.setLayout(layout)
        self.setStyleSheet(_S)
        self.__dict__['spinBox'] = spin = TaurusValueSpinBox(
            qt_parent=self, designMode=designMode)
        self.__dict__['sliderButton1'] = b1 = Qt.QToolButton(self)
        self.__dict__['sliderButton2'] = b2 = Qt.QToolButton(self)
        b1.setIcon(getStandardIcon(Qt.QStyle.SP_TitleBarShadeButton, b1))
        b2.setIcon(getStandardIcon(Qt.QStyle.SP_TitleBarUnshadeButton, b2))
        layout.addWidget(spin, 0, 0, 2, 1)
        layout.addWidget(b1, 0, 1, 1, 1, Qt.Qt.AlignBottom)
        layout.addWidget(b2, 1, 1, 1, 1, Qt.Qt.AlignTop)
        layout.setColumnStretch(0, 1)
        layout.setColumnStretch(1, 0)

        policy = self.sizePolicy()
        policy.setHorizontalPolicy(Qt.QSizePolicy.Minimum)
        policy.setVerticalPolicy(Qt.QSizePolicy.Fixed)
        self.setSizePolicy(policy)

    def __getattr__(self, name):
        return getattr(self.spinBox, name)

    def __setattr__(self, name, value):
        setattr(self.spinBox, name, value)



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
###########################################################################

"""This package provides a dialog for graphically choosing a Taurus class"""

__all__ = ["GraphicalChoiceDlg", "GraphicalChoiceWidget"]

__docformat__ = 'restructuredtext'

from taurus.external.qt import Qt


class GraphicalChoiceDlg(Qt.QDialog):
    '''
    A generic dialog for choosing among a set of choices which are presented as
    an array of, each with a given pixmap.

    The :meth:`getChoice` static method is provided for convenience so that the
    dialog can be invoked wit a single line::

        chosen,ok = GraphicalChoiceDlg.getChoice(parent, title, msg, choices, pixmaps, size, defpixmap, horizontalScrollBarPolicy, verticalScrollBarPolicy)
    '''

    def __init__(self, parent=None, designMode=False, choices=None, pixmaps=None, iconSize=128, defaultPixmap=None, horizontalScrollBarPolicy=Qt.Qt.ScrollBarAsNeeded, verticalScrollBarPolicy=Qt.Qt.ScrollBarAsNeeded):
        Qt.QDialog.__init__(self, parent)
        self._chosen = None
        self.setLayout(Qt.QVBoxLayout())
        self.label = Qt.QLabel('Click on your choice:')
        self.layout().addWidget(self.label)
        self._iconsArea = GraphicalChoiceWidget(parent=parent, designMode=designMode, choices=choices,
                                                pixmaps=pixmaps, iconSize=iconSize, defaultPixmap=defaultPixmap,
                                                horizontalScrollBarPolicy=horizontalScrollBarPolicy,
                                                verticalScrollBarPolicy=verticalScrollBarPolicy)
        self.layout().addWidget(self._iconsArea)
        self._iconsArea.choiceMade.connect(self.onChoiceMade)

    def setHorizontalScrollBarPolicy(self, policy):
        '''sets horizontal scrollbar policy of scrollArea'''
        return self._iconsArea.setHorizontalScrollBarPolicy(policy)

    def setVerticalScrollBarPolicy(self, policy):
        '''sets vertical scrollbar policy of scrollArea'''
        return self._iconsArea.setVerticalScrollBarPolicy(policy)

    def setMessage(self, msg):
        '''sets the text which is shown to the user in the dialog'''
        self.label.setText(msg)

    def onChoiceMade(self, chosen):
        '''slot called when the user chooses an option'''
        self.accept()

    def getChosen(self):
        '''
        returns the choice
        :return: (str)
        '''
        return self._iconsArea.getChosen()

    @staticmethod
    def getChoice(parent=None, title='', msg='', choices=None, pixmaps=None, iconSize=128, defaultPixmap=None, horizontalScrollBarPolicy=Qt.Qt.ScrollBarAsNeeded, verticalScrollBarPolicy=Qt.Qt.ScrollBarAsNeeded):
        '''
        Static method which launches a GraphicalChoiceDlg with the given options
        and returns the result

        :param parent: (QWidget) The parent of the dialog (it will be centered on it)
        :param title: (str) the text which is displayed in the title bar of the dialog
        :param msg: (str) the text which is shown to the user in the dialog,
                    above the choices.
        :param choices: (list<list>) a list of lists of strings to be used as
                        choices names. The (possibly sparse) 2D array defined by
                        the nested lists will be used to present the choices in
                        a grid. The choice names will be used as keys for pixmaps
        :param pixmaps: (dict<str,QPixmap>) dictionary mapping the choices text to
                        corresponding pixmap. If no valid pixmap is provided for
                        a given choice, the defaultPixmap will be used
        :param iconSize: (int) size of the icons to be displayed (128px by default)
        :param defaultPixmap: (QPixmap) Default Pixmap to use if none passed for a
                              given choice. No Pixmap will be used if None passed.
        :param horizontalScrollBarPolicy: (enum Qt.ScrollBarPolicy) defines the mode of the horizontal scroll bar.
                                          The default mode is ScrollBarAsNeeded.
        :param verticalScrollBarPolicy: (enum Qt.ScrollBarPolicy) defines the mode of the vertical scroll bar.
                                        The default mode is ScrollBarAsNeeded
        :return: (tuple<str,bool>) A tuple containing choice,ok. choice is the name of
                 the chosen option. ok is true if the user pressed OK and false
                 if the user pressed Cancel.
        '''
        dlg = GraphicalChoiceDlg(parent=parent, choices=choices, pixmaps=pixmaps, iconSize=iconSize,
                                 defaultPixmap=defaultPixmap, horizontalScrollBarPolicy=horizontalScrollBarPolicy,
                                 verticalScrollBarPolicy=verticalScrollBarPolicy)
        dlg.setWindowTitle(title)
        dlg.setMessage(msg)
        dlg.exec_()
        return dlg.getChosen(), (dlg.result() == dlg.Accepted)


class GraphicalChoiceWidget(Qt.QScrollArea):
    '''A widget that presents a 2D grid of buttons'''

    choiceMade = Qt.pyqtSignal(str)

    def __init__(self, parent=None, designMode=False, choices=None, pixmaps=None, iconSize=128,
                 defaultPixmap=None, horizontalScrollBarPolicy=Qt.Qt.ScrollBarAsNeeded,
                 verticalScrollBarPolicy=Qt.Qt.ScrollBarAsNeeded):
        Qt.QScrollArea.__init__(self, parent)

        self._chosen = None
        self._iconSize = iconSize
        if defaultPixmap is None:
            defaultPixmap = Qt.QPixmap()
        self._defaultPixmap = defaultPixmap

        self.setFrameShape(Qt.QFrame.NoFrame)
        self.setWidgetResizable(True)
        self.setHorizontalScrollBarPolicy(horizontalScrollBarPolicy)
        self.setVerticalScrollBarPolicy(verticalScrollBarPolicy)

        w = Qt.QWidget()
        self.gridLayout = Qt.QGridLayout()
        w.setLayout(self.gridLayout)
        self.setWidget(w)

        if choices is not None:
            self.setChoices(choices, pixmaps)
        elif designMode:
            from taurus.qt.qtgui.icon import getCachedPixmap
            pm = getCachedPixmap('logos:taurus.png')
            self.setChoices([['choice1', 'choice2'], ['choice3', 'choice4']],
                            dict(choice1=pm, choice2=pm, choice3=pm,
                                 choice4=pm))

    def setChoices(self, choices, pixmaps=None):
        '''
        sets the available options

        :param choices: (list<list>) a list of lists of strings to be used as
                        choices names. The (possibly sparse) 2D array defined by
                        the nested lists will be used to present the choices in
                        a grid. The choice names will be used as keys for pixmaps
        :param pixmaps: (dict<str,QPixmap>) dictionary mapping the choices text to
                        corresponding pixmap. If no valid pixmap is provided for
                        a given choice, a default pixmap will be used
        '''
        if pixmaps is None:
            pixmaps = {}
        for i, rowlist in enumerate(choices):
            for j, choice in enumerate(rowlist):
                self.setChoice(i, j, str(choice),
                               pixmap=pixmaps.get(choice, None))

    def setChoice(self, row, col, text, pixmap=None, tooltip=None):
        '''
        sets the option for a given row,column coordinate in the grid

        :param row: (int) row in the grid for this option
        :param col: (int) column in the grid for this option
        :param text: (str) name for this option
        :param pixmap: (QPixmap or None) If no valid pixmap is provided for
                        a given choice, the default one will be used
        :param tooltip: (str) tooltip for this option (if None given, the `text` is used)
        '''
        if not pixmap or pixmap is None or pixmap.isNull():
            pixmap = self._defaultPixmap
        if tooltip is None:
            tooltip = text
        button = Qt.QToolButton()
        button.setText(text)
        button.setIcon(Qt.QIcon(pixmap))
        button.setIconSize(Qt.QSize(self._iconSize, self._iconSize))
        button.setToolTip(tooltip)
        button.clicked.connect(self.onClick)
        self.gridLayout.addWidget(button, row, col, Qt.Qt.AlignCenter)

    def onClick(self):
        '''slot called when a button is clicked'''
        self._chosen = unicode(self.sender().text())
        self.choiceMade.emit(self._chosen)

    def getChosen(self):
        '''
        returns the choice
        :return: (str)
        '''
        return self._chosen

    @classmethod
    def getQtDesignerPluginInfo(cls):
        """Returns pertinent information in order to be able to build a valid
        QtDesigner widget plugin

        The dictionary returned by this method should contain *at least* the
        following keys and values:
        - 'module' : a string representing the full python module name (ex.: 'taurus.qt.qtgui.base')
        - 'icon' : a string representing valid resource icon (ex.: 'designer:combobox.png')
        - 'container' : a bool telling if this widget is a container widget or not.

        This default implementation returns the following dictionary::

            { 'group'     : 'Taurus Widgets',
              'icon'      : 'logos:taurus.png',
              'container' : False }

        :return: (dict) a map with pertinent designer information"""
        return {
            'module': 'taurus.qt.qtgui.input',
            'group': 'Taurus Input',
            'icon': 'logos:taurus.png',
            'container': False}


#------------------------------------------------------------------------------

def testWidget():
    import sys
    app = Qt.QApplication(sys.argv)
    w = GraphicalChoiceWidget(None, True)
    w.show()
    sys.exit(app.exec_())


def main():
    import sys
    from taurus.qt.qtgui.icon import getCachedPixmap
    app = Qt.QApplication(sys.argv)

    pixmaps = {}
    choices = [['TaurusForm', 'TaurusTrend'], ['TaurusPlot', 'Qub']]
    for row in choices:
        for k in row:
            pixmaps[k] = getCachedPixmap('snapshot:%s.png' % k)

    print GraphicalChoiceDlg.getChoice(parent=None, title='Panel chooser', msg='Choose the type of Panel:', choices=choices, pixmaps=pixmaps)

    sys.exit()


if __name__ == "__main__":
    main()
    # testWidget()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""Test for taurus.core.evaluation.test.test_evalvalidator..."""


__docformat__ = 'restructuredtext'

import taurus
from taurus.external import unittest
from taurus.core.test import (valid, invalid, names,
                              AbstractNameValidatorTestCase)
from taurus.core.evaluation.evalvalidator import (EvaluationAuthorityNameValidator,
                                                  EvaluationDeviceNameValidator,
                                                  EvaluationAttributeNameValidator)

try:
    authority = taurus.Factory('tango').getAuthority()
    default_tango_authority = authority.getSimpleName()
    reason = None
except:
    default_tango_authority = None
    reason = "Tango scheme is not available"
#=========================================================================
# Tests for Eval Authority  name validation
#=========================================================================


@valid(name='eval://localhost')
@invalid(name='eval://foo:10000')
@invalid(name='eval://user@foo:10000')
@invalid(name='eval://user@localhost')
@invalid(name='eval://localhost/')
@invalid(name='eval://localhost/?')
@invalid(name='eval:foo')
@valid(name='eval://db=foo', strict=False)
@invalid(name='eval://db=foo', strict=True)
@valid(name='//db=foo', strict=False)  # using implicit scheme!
@names(name='eval://localhost',
       out=('eval://localhost', '//localhost', 'localhost'))
@names(name='//localhost',  # using implicit scheme!
       out=('eval://localhost', '//localhost', 'localhost'))
class EvaluationAuthValidatorTestCase(AbstractNameValidatorTestCase,
                                      unittest.TestCase):
    validator = EvaluationAuthorityNameValidator


#=========================================================================
# Tests for Eval Device name validation
#=========================================================================
@invalid(name='eval:foo')
@valid(name='eval:@foo')
@valid(name='eval:@mymod.Myclass', groups={'devname': '@mymod.Myclass'})
@valid(name='eval:@mymod.mysubmod.Myclass')
@valid(name='eval:/@foo', groups={'devname': '@foo', 'path': '/@foo'})
@valid(name='eval://localhost/@foo')
@valid(name='eval:@ foo', groups={'devname': '@ foo'})
@invalid(name='eval:@foo?')
@valid(name='eval://dev=foo', strict=False)
@valid(name='eval://db=localhost;dev=foo', strict=False)
@invalid(name='eval://@foo', strict=False)
@invalid(name='eval://db=localhost;@foo', strict=False)
@invalid(name='eval://localhost;dev=foo', strict=False)
@valid(name='eval:@foo')
@invalid(name='eval://a/b', strict=True)
@invalid(name='eval://d')
@names(name='eval://localhost/@foo',
       out=('eval://localhost/@foo', '@foo', 'foo'))
@names(name='eval://dev=foo',
       out=('eval://localhost/@foo', '@foo', 'foo'))
class EvaluationDevValidatorTestCase(AbstractNameValidatorTestCase,
                                     unittest.TestCase):
    validator = EvaluationDeviceNameValidator


#=========================================================================
# Tests for Eval Attribute name validation
#=========================================================================

@valid(name='eval://db=localhost;dev=taurus.core.evaluation.dev_example.FreeSpaceDevice;getFreeSpace("/")/1024/1024',
       strict=False)
@valid(name='eval:1')
#=========================================================================
# this is equivalent to "eval:1". The '/' is part of the path, but not of the
# attrname
@valid(name='eval:/1', groups={'path': '/1', 'attrname': '1'})
#=========================================================================
@valid(name='eval:1/')  # "1/" is wrong math, but valid as an attr name!
@valid(name='eval:1/3')
@valid(name='eval:@foo/1')
@valid(name='eval:@mymod.Myclass/1.2',
       groups={'attrname': '1.2', 'devname': '@mymod.Myclass'})
@valid(name='eval://linspace(-1, 1, 256)',
       groups={'attrname': 'linspace(-1, 1, 256)',
               '_expr': 'linspace(-1, 1, 256)',
               '_subst': None,
               'fragment': None,
               '__STRICT__': False,
               'cfgkey': None}, strict=False)
@valid(name="eval://rand(256, 128)",
       groups={'attrname': 'rand(256, 128)',
               '_expr': 'rand(256, 128)',
               '_subst': None,
               'fragment': None,
               '__STRICT__': False,
               'cfgkey': None}, strict=False)
@valid(name="eval:rand(256, 128)",
       groups={'attrname': 'rand(256, 128)',
               '_expr': 'rand(256, 128)',
               '_subst': None,
               'fragment': None,
               '__STRICT__': True,
               'cfgkey': None}, strict=True)
@valid(name="eval:rand(256, 128)#label",
       groups={'attrname': 'rand(256, 128)',
               '_expr': 'rand(256, 128)',
               '_subst': None,
               'fragment': 'label',
               '__STRICT__': True,
               'cfgkey': 'label'}, strict=True)
@valid(name="eval:foo=10;rand(256, 128)+foo#label",
       groups={'attrname': 'foo=10;rand(256, 128)+foo',
               '_expr': 'rand(256, 128)+foo',
               '_subst': 'foo=10;',
               'fragment': 'label',
               '__STRICT__': True,
               'cfgkey': 'label'}, strict=True)
@valid(name="eval://rand(256, 128)?configuration=label",
       groups={'attrname': 'rand(256, 128)',
               '_expr': 'rand(256, 128)',
               '_subst': None,
               'query': 'configuration=label',
               'fragment': 'label',
               '__STRICT__': False,
               'cfgkey': 'label'}, strict=False)
@valid(name="eval:foo=1;bar=2;tar=3;foo+bar+tar#label",
       groups={'attrname': 'foo=1;bar=2;tar=3;foo+bar+tar',
               '_expr': 'foo+bar+tar',
               '_subst': 'foo=1;bar=2;tar=3;',
               'query': None,
               'fragment': 'label',
               '__STRICT__': True,
               'cfgkey': 'label'}, strict=True)
@valid(name='eval:a={eval:1#f};a+{eval:2#foo}#bar',
       groups={'attrname': 'a={eval:1#f};a+{eval:2#foo}',
               '_expr': 'a+{eval:2#foo}',
               '_subst': 'a={eval:1#f};',
               'query': None,
               'fragment': 'bar',
               '_evalrefs': ['eval:1#f', 'eval:2#foo'],
               '__STRICT__': True,
               'cfgkey': 'bar'}, strict=True)
@valid(name='eval:@foo/1/3')
@valid(name='eval:x=2;y=3;x*y')
@valid(name='eval:@foo/x=2;y=3;x*y')
@valid(name='eval:{tango:a/b/c/d}')
@valid(name='eval:a={tango:a/b/c/d};x=2;a*x',
       groups={'attrname': 'a={tango:a/b/c/d};x=2;a*x',
               '_expr': 'a*x',
               '_subst': 'a={tango:a/b/c/d};x=2;',
               '_evalrefs': ['tango:a/b/c/d'],
               'fragment': None})
@valid(name='eval://dev=foo;1', strict=False)
@valid(name='eval://dev=foo;1/3', strict=False)
@valid(name='eval://1/3', strict=False)
@valid(name='eval://dev=foo;x*y?x=2;y=3', strict=False)
@valid(name='eval://x+y+z?x=1;y=2;z=3', strict=False)
@valid(name='eval://a*x?a={tango://a/b/c/d};x=2', strict=False)
@invalid(name='eval://dev=foo;1')
@invalid(name='eval://dev=foo;1/3')
@invalid(name='eval://1/3')
@invalid(name='eval://dev=foo;x*y?x=2;y=3')
@invalid(name='eval://x+y+z?x=1;y=2;z=3')
@invalid(name='eval://a*x?a={tango://a/b/c/d};x=2')
# TODO #1 maybe we can check if the substitutions can be done
@valid(name='eval://configuration?configuration=1', strict=False)
@valid(name='eval://config?config=1', strict=False)
@valid(name='eval://configurationfoo?configurationfoo=1', strict=False)
@valid(name='eval://foo?bar=1', strict=False)  # TODO #1
@valid(name='eval:bar=1;foo', strict=False)  # TODO #1
@invalid(name='eval://?configurationfoo=1', strict=False)
@invalid(name='eval://?configuration=1', strict=False)
@valid(name='eval://1?configuration=foo1', strict=False)
@valid(name='eval://a?a=1?configuration=label', strict=False)
@valid(name='eval://a+b?a=1;b=2?configuration=label', strict=False)
@valid(name='eval://1?a=1?configuration=label', strict=False)
@valid(name='eval://1?configuration', strict=False)
@valid(name='eval://1?configuration=', strict=False)
@invalid(name='eval://@foo/1')
@valid(name='eval:/@foo/1', groups={'path': '/@foo/1', 'attrname': '1'})
@valid(name='eval:@foo/1', groups={'path': '@foo/1', 'attrname': '1'})
# @invalid(name='eval:1#label')
#
# @invalid(name='eval:1?foo')
# @invalid(name='eval:1?configuration')
# @invalid(name='eval:1?configuration=foo')
@invalid(name='eval:@foo')  # invalid because is a device URI
@invalid(name='eval:{tango:a/b/c}')  # invalid because is a device URI
#=========================================================================
#
# this should be invalid because the reference is a dev name instead of an
# attr name. But for the moment we do not check such details...
# #TODO: It could be implemented by using
#        isValidName(ref, etypes=TaurusElementType.Attribute)
#=========================================================================
@names(name='eval:1', out=('eval://localhost/@DefaultEvaluator/1', '1', '1'))
@names(name='eval:@Foo/1', out=('eval://localhost/@Foo/1', '@Foo/1', '1'))
@names(name='eval:a={tango:a/b/c/d};x=2;a*x',
       out=('eval://localhost/@DefaultEvaluator/' +
            'a={tango://%s/a/b/c/d};x=2;a*x' % default_tango_authority,
            'a={tango:a/b/c/d};x=2;a*x',
            'a*x'), test_skip=reason)
@names(name='eval:@Foo/c=1;cos(0)+c',
       out=('eval://localhost/@Foo/c=1;cos(0)+c',
            '@Foo/c=1;cos(0)+c',
            'cos(0)+c'))
@names(name='eval:a={eval:1#f};a+{eval:2#foo}#bar',
       out=('eval://localhost/@DefaultEvaluator/a=' +
            '{eval://localhost/@DefaultEvaluator/1};a+' +
            '{eval://localhost/@DefaultEvaluator/2}',
            'a={eval:1#f};a+{eval:2#foo}',
            'a+2'))
@names(name='eval:a={eval:"a"};{eval:0}*a*{eval:1+{eval:a=2;a*a}}',
       out=('eval://localhost/@DefaultEvaluator/' +
            'a={eval://localhost/@DefaultEvaluator/"a"};' +
            '{eval://localhost/@DefaultEvaluator/0}*a*' +
            '{eval://localhost/@DefaultEvaluator/1+' +
            '{eval://localhost/@DefaultEvaluator/a=2;a*a}}',
            'a={eval:"a"};{eval:0}*a*{eval:1+{eval:a=2;a*a}}',
            '0*a*1+a*a'))
@names(name='eval:@Foo/{NonExistingAlias/foo}+5',
       out=(None,
            '@Foo/{NonExistingAlias/foo}+5',
            'foo+5'))
@names(name='eval:{eval:1}+{eval:2}',
       out=('eval://localhost/@DefaultEvaluator/' +
            '{eval://localhost/@DefaultEvaluator/1}+' +
            '{eval://localhost/@DefaultEvaluator/2}',
            '{eval:1}+{eval:2}',
            '1+2'))
@names(name='eval:{eval:{eval:foo=1;foo}}',
       out=('eval://localhost/@DefaultEvaluator/{eval://localhost/' +
            '@DefaultEvaluator/{eval://localhost/@DefaultEvaluator/foo=1;foo}}',
            '{eval:{eval:foo=1;foo}}',
            'foo'))
@names(name='eval:x=0;x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+{w/x/y/x}*x*{eval:x}',
       out=('eval://localhost/@DefaultEvaluator/' +
            'x=0;x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+' +
            '{tango://%s/w/x/y/x}*x*' % default_tango_authority +
            '{eval://localhost/@DefaultEvaluator/x}',
            'x=0;x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+{w/x/y/x}*x*{eval:x}',
            'x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+x*x*x'), test_skip=reason)
# old syntax gets transformed into new one!
@names(name='eval://dev=foo;x*y?x=2;y=3',
       out=('eval://localhost/@foo/x=2;y=3;x*y',
            '@foo/x=2;y=3;x*y',
            'x*y'))
# old syntax gets transformed into new one!
@names(name='eval://x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+{w/x/y/x}*x*{eval:x}?x=0',
       out=('eval://localhost/@DefaultEvaluator/' +
            'x=0;x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+' +
            '{tango://%s/w/x/y/x}*x*' % default_tango_authority +
            '{eval://localhost/@DefaultEvaluator/x}',
            'x=0;x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+{w/x/y/x}*x*{eval:x}',
            'x+x-x*x/x+cos(x)-xxx*xy+yx+yxz+x+x*x*x'), test_skip=reason)
@valid(name='eval:1*{eval:x=3;x}')
@valid(name='eval:1*{eval:2*{eval:3}}')
@valid(name='eval:{eval:@foo/3}')
@valid(name='eval:a={tango:a/b/c/d};{eval:0}+a+{eval:1+{eval:a=2;a*a}}',
       groups={'_evalrefs': ['tango:a/b/c/d', 'eval:0', 'eval:1+{eval:a=2;a*a}']})
@valid(name='eval:a="{eval:0}";b={eval:1};a*b+"{eval:2}"*{eval:3}#{attr.label}',
       groups={'_evalrefs': ['eval:1', 'eval:3']})
@valid(name='eval:a="{eval:1}";b={eval:1};a*b+"{eval:3}"*{eval:3}#{attr.label}',
       groups={'_evalrefs': ['eval:1', 'eval:3']})
@valid(name='eval:"{eval:1}"+"{eval:1}"*{eval:1}*{eval:3}',
       groups={'_evalrefs': ['eval:1', 'eval:3']})
@valid(name='eval:"{eval:1}"*{eval:1}+"{eval:1}"*{eval:3}',
       groups={'_evalrefs': ['eval:1', 'eval:3']})
@valid(name='eval:{eval:1}*"{eval:1}"+"{eval:1}"*{eval:3}',
       groups={'_evalrefs': ['eval:1', 'eval:3']})
# invalid because of non-strict ref
@invalid(name='eval:1*{eval:2*{eval://3}}')
@valid(name='eval:1*{eval:2*{eval://3}}', strict=False)
@invalid(name='eval:{tango:a/b/c}')  # invalid:  ref is not an attr!
# invalid: ref is not an attr!
@invalid(name='eval:{tango:a/b/c}', strict=False)
@valid(name='eval:{tango:a/b/c/d}')
@invalid(name='eval:{tango://a/b/c/d}')  # invalid because of non-strict ref
# but valid with old syntax
@valid(name='eval:{tango://a/b/c/d}', strict=False)
#=========================================================================
# Tests for eval attribute  name validation (when passing fragment / cfgkey)
#=========================================================================
@valid(name='eval:1#')
@valid(name='eval:1#units', groups={'fragment': 'units'})
@valid(name='eval:{tango:a/b/c/d}*2#')
@valid(name='eval:{tango:a/b/c/d}*2#label')
@valid(name='eval:k=2;a={tango:a/b/c/d};a*k#units')
@valid(name='eval://localhost/@Foo/k=2;a={eval:1};a*k#label')
@valid(name='eval://localhost/@mymod.MyEvalClass/1#label')
@invalid(name='eval:1# ')  # invalid because of the trailing space
@names(name='eval:1#units',
       out=('eval://localhost/@DefaultEvaluator/1',
            '1',
            '1', 'units'))
@names(name='eval:@Foo/a={tango:a/b/c/d};x=2;a*x#',
       out=('eval://localhost/@Foo/a={tango://%s/a/b/c/d};x=2;a*x' %
            default_tango_authority,
            '@Foo/a={tango:a/b/c/d};x=2;a*x',
            'a*x', ''), test_skip=reason)
# old syntax gets transformed into new one!
@names(name='eval://1?configuration=units',
       out=('eval://localhost/@DefaultEvaluator/1',
            '1',
            '1', 'units'))
# old syntax gets transformed into new one!
@names(name='eval://dev=Foo;a*x?a={tango:a/b/c/d};x=2?configuration',
       out=('eval://localhost/@Foo/a={tango://%s/a/b/c/d};x=2;a*x' %
            default_tango_authority,
            '@Foo/a={tango:a/b/c/d};x=2;a*x',
            'a*x', None), test_skip=reason)
# old syntax gets transformed into new one!
@names(name='eval://dev=Foo;a*x?a={tango:a/b/c/d};x=2?configuration=',
       out=('eval://localhost/@Foo/a={tango://%s/a/b/c/d};x=2;a*x' %
            default_tango_authority,
            '@Foo/a={tango:a/b/c/d};x=2;a*x',
            'a*x', ''), test_skip=reason)
# old syntax gets transformed into new one!
@names(name='eval://dev=Foo;a*x?a={tango:a/b/c/d};x=2?configuration=label',
       out=('eval://localhost/@Foo/a={tango://%s/a/b/c/d};x=2;a*x' %
            default_tango_authority,
            '@Foo/a={tango:a/b/c/d};x=2;a*x',
            'a*x', 'label'), test_skip=reason)
class EvaluationAttrValidatorTestCase(AbstractNameValidatorTestCase,
                                      unittest.TestCase):
    validator = EvaluationAttributeNameValidator


if __name__ == '__main__':
    pass



# -*- coding: utf-8 -*-

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""utilities to load ui files for widgets"""

__all__ = ["loadUi",
           "UILoadable",
           ]

import os
import sys
import functools

from taurus.external.qt import Qt
from taurus.external.qt import uic


class __UI(object):
    pass


def loadUi(obj, filename=None, path=None, with_ui=None):
    """
    Loads a QtDesigner .ui file into the given widget.
    If no filename is given, it tries to load from a file name which is the
    widget class name plus the extension ".ui" (example: if your
    widget class is called MyWidget it tries to find a MyWidget.ui).
    If path is not given it uses the directory where the python file which
    defines the widget is located plus a *ui* directory (example: if your widget
    is defined in a file /home/homer/workspace/taurusgui/my_widget.py then it uses
    the path /home/homer/workspace/taurusgui/ui)

    :param filename: the QtDesigner .ui file name [default: None, meaning
                      calculate file name with the algorithm explained before]
    :type filename: str
    :param path: directory where the QtDesigner .ui file is located
                 [default: None, meaning calculate path with algorithm explained
                 before]
    :type path: str
    :param with_ui: if True, the objects defined in the ui file will be
                    accessible as submembers of an ui member of the widget. If
                    False, such objects will directly be members of the widget.
    :type with_ui: bool
    """
    if path is None:
        obj_file = sys.modules[obj.__module__].__file__
        path = os.path.join(os.path.dirname(obj_file), 'ui')
    if filename is None:
        filename = obj.__class__.__name__ + os.path.extsep + 'ui'
    full_name = os.path.join(path, filename)

    if with_ui is not None:
        ui_obj = __UI()
        setattr(obj, with_ui, ui_obj)
        previous_members = set(dir(obj))

        uic.loadUi(full_name, baseinstance=obj)

        post_members = set(dir(obj))
        new_members = post_members.difference(previous_members)
        for member_name in new_members:
            member = getattr(obj, member_name)
            setattr(ui_obj, member_name, member)
            delattr(obj, member_name)
    else:
        uic.loadUi(full_name, baseinstance=obj)


def UILoadable(klass=None, with_ui=None):
    """
    A class decorator intended to be used in a Qt.QWidget to make its UI
    loadable from a predefined QtDesigner UI file.
    This decorator will add a :func:`loadUi` method to the decorated class and
    optionaly a property with a name given by *with_ui* parameter.

    The folowing example assumes the existence of the ui file
    :file:`<my_widget_dir>/ui/MyWidget.ui` which is a QWidget panel with *at
    least* a QPushButton with objectName *my_button* ::

        from taurus.external.qt import Qt
        from taurus.qt.qtgui.util.ui import UILoadable

        @UILoadable
        class MyWidget(Qt.QWidget):

            def __init__(self, parent=None):
                Qt.QWidget.__init__(self, parent)
                self.loadUi()
                self.my_button.setText("This is MY button")

    Another example using a :file:`superUI.ui` file in the same directory as
    the widget. The widget UI components can be accessed through the widget
    member *_ui* ::

        import os.path

        from taurus.external.qt import Qt
        from taurus.qt.qtgui.util.ui import UILoadable

        @UILoadable(with_ui="_ui")
        class MyWidget(Qt.QWidget):

            def __init__(self, parent=None):
                Qt.QWidget.__init__(self, parent)
                self.loadUi(filename="superUI.ui", path=os.path.dirname(__file__))
                self._ui.my_button.setText("This is MY button")

    :param with_ui: assigns a member to the decorated class from which you
                    can access all UI components [default: None, meaning no
                    member is created]
    :type with_ui: str

    .. warning::
        the current implementation (Jul14) doesn't prevent Qt from overloading
        any members you might have defined previously by the widget object names
        from the UI file. This happens even if *with_ui* parameter is given.
        For example, if the UI contains a QPushButton with objectName
        *my_button*::

            @UILoadable(with_ui="_ui")
            class MyWidget(Qt.QWidget):

                def __init__(self, parent=None):
                    Qt.QWidget.__init__(self, parent)
                    self.my_button = "hello"
                    self.loadUi()
            widget = MyWidget()
            print widget.my_button
            <PyQt4.QtGui.QPushButton object at 0x159e2f8>

        This little problem should be solved in the next taurus version.
    """
    if klass is None:
        return functools.partial(UILoadable, with_ui=with_ui)

    klass_name = klass.__name__
    klass_file = sys.modules[klass.__module__].__file__
    klass_path = os.path.join(os.path.dirname(klass_file), 'ui')

    def _loadUi(self, filename=None, path=None):
        if filename is None:
            filename = klass_name + os.path.extsep + 'ui'
        if path is None:
            path = klass_path
        return loadUi(self, filename=filename, path=path, with_ui=with_ui)

    klass.loadUi = _loadUi
    return klass


def main():
    from taurus.qt.qtgui.application import TaurusApplication

    app = TaurusApplication([])

    @UILoadable(with_ui="ui")
    class A(Qt.QWidget):

        def __init__(self, parent=None):
            Qt.QWidget.__init__(self, parent)
            import taurus.qt.qtgui.panel.ui
            path = os.path.dirname(taurus.qt.qtgui.panel.ui.__file__)
            self.loadUi(filename='TaurusMessagePanel.ui', path=path)

    gui = A()
    gui.show()
    app.exec_()

if __name__ == "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module contains all basic tango enumerations"""

__all__ = ["TangoObjectType", "EVENT_TO_POLLING_EXCEPTIONS",
           "FROM_TANGO_TO_NUMPY_TYPE", "FROM_TANGO_TO_STR_TYPE", "DevState"]

__docformat__ = "restructuredtext"

from taurus.core.util.enumeration import Enumeration
from taurus.external.enum import IntEnum

TangoObjectType = Enumeration("TangoObjectType",
                              ["Authority", "Server", "Class", "Device",
                               "Attribute", "Property", "Configuration",
                               "Object"])
TangoObjectType.Database = TangoObjectType.Authority  # backwards compatibility

import numpy
import PyTango

# The exception reasons that will force switching from events to polling
# API_AttributePollingNotStarted - the attribute does not support events.
#                                  Don't try to resubscribe.
# API_DSFailedRegisteringEvent - same exception then the one above but higher
#                                in the stack
# API_NotificationServiceFailed - Problems in notifd, it was not able to
#                                 register the event.
# API_EventChannelNotExported - the notifd is not running
# API_EventTimeout - after a successfull register the the device server
#                    and/or notifd shuts down/crashes
# API_CommandNotFound - Added on request from ESRF (Matias Guijarro). They have
#                       a DS in java (doesn't have events) and the only way they
#                       found to fix the event problem was to add this exception
#                       type here. Maybe in future this will be solved in a better
#                       way
# API_BadConfigurationProperty - the device server is not running
EVENT_TO_POLLING_EXCEPTIONS = ('API_AttributePollingNotStarted',
                               'API_DSFailedRegisteringEvent',
                               'API_NotificationServiceFailed',
                               'API_EventChannelNotExported',
                               'API_EventTimeout',
                               'API_EventPropertiesNotSet',
                               'API_CommandNotFound',
                               )
#                                   'API_BadConfigurationProperty')

FROM_TANGO_TO_NUMPY_TYPE = {
    PyTango.DevBoolean: numpy.bool8,
    PyTango.DevUChar: numpy.uint8,
    PyTango.DevShort: numpy.short,
    PyTango.DevUShort: numpy.ushort,
    PyTango.DevLong: numpy.int32,
    PyTango.DevULong: numpy.uint32,
    PyTango.DevLong64: numpy.int64,
    PyTango.DevULong64: numpy.uint64,
    PyTango.DevString: numpy.str,
    PyTango.DevDouble: numpy.float64,
    PyTango.DevFloat: numpy.float32,
}

FROM_TANGO_TO_STR_TYPE = {
    PyTango.DevBoolean: 'bool8',
    PyTango.DevUChar: 'uint8',
    PyTango.DevShort: 'short',
    PyTango.DevUShort: 'ushort',
    PyTango.DevLong: 'int32',
    PyTango.DevULong: 'uint32',
    PyTango.DevLong64: 'int64',
    PyTango.DevULong64: 'uint64',
    PyTango.DevString: 'str',
    PyTango.DevDouble: 'float64',
    PyTango.DevFloat: 'float32',
}


class DevState(IntEnum):
    """ This is the taurus.core.tango equivalent to PyTango.DevState.
    It defines the same members and uses the same numerical values internally,
    allowing equality comparisons with :class:`PyTango.DevState` (but not
    identity checks!)::

        from taurus.core.tango import DevState as D1
        from PyTango import DevState as D2

        D1.OPEN == D2.OPEN          # --> True
        D1.OPEN in (D2.ON, D2.OPEN) # --> True
        D1.OPEN == 3                # --> True
        D1.OPEN is 3                # --> False
        D1.OPEN is D2.OPEN          # --> False

     """
    ON = 0
    OFF = 1
    CLOSE = 2
    OPEN = 3
    INSERT = 4
    EXTRACT = 5
    MOVING = 6
    STANDBY = 7
    FAULT = 8
    INIT = 9
    RUNNING = 10
    ALARM = 11
    DISABLE = 12
    UNKNOWN = 13



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

""" """

__all__ = ["List"]

__docformat__ = "restructuredtext"

import textwrap
import operator

from enums import Alignment


class List(list):

    HeaderSeparator = "-"
    RowSeparator = None

    MaxColumnWidth = -1
    TextAlignment = Alignment.Right | Alignment.Top

    def __init__(self, header, header_separator=HeaderSeparator,
                 row_separator=RowSeparator,
                 max_col_width=MaxColumnWidth,
                 text_alignment=TextAlignment):

        self.col_nb = col_nb = len(header)
        self.cur_col_width = col_nb * [0]

        self.header_separator = header_separator
        self.row_separator = row_separator
        self.max_column_width = max_col_width
        self.text_alignment = text_alignment

        if header is not None:
            self.append(header)

    def setHeaderSeparator(self, header_separator):
        if isinstance(header_separator, (str, unicode)):
            header_separator = self.col_nb * [header_separator]
        self.HeaderSeparator = header_separator

    def getHeaderSeparator(self):
        return self.HeaderSeparator

    header_separator = property(getHeaderSeparator, setHeaderSeparator)

    def setRowSeparator(self, row_separator):
        if isinstance(row_separator, (str, unicode)):
            row_separator = self.col_nb * [row_separator]
        self.RowSeparator = row_separator

    def getRowSeparator(self):
        return self.RowSeparator

    row_separator = property(getRowSeparator, setRowSeparator)

    def setMaxColumnWidth(self, max_col_width):
        if max_col_width is None:
            max_col_width = -1
        if not operator.isSequenceType(max_col_width):
            max_col_width = self.col_nb * [max_col_width]
        self.MaxColumnWidth = max_col_width

    def getMaxColumnWidth(self):
        return self.MaxColumnWidth

    max_column_width = property(getMaxColumnWidth, setMaxColumnWidth)

    def setTextAlignment(self, text_alignment):
        if not operator.isSequenceType(text_alignment):
            text_alignment = self.col_nb * [text_alignment]
        self.TextAlignment = text_alignment

    def getTextAlignment(self):
        return self.TextAlignment

    text_alignment = property(getTextAlignment, setTextAlignment)

    def _transform_row(self, row):
        return map(str, row[:self.col_nb])

    def __setitem__(self, i, row):
        return list.__setitem__(self, i, self._transform_row(row))

    def append(self, row):
        return list.append(self, self._transform_row(row))

    appendRow = append

    def putRow(self, row, idx):
        self[idx] = row

    def _calc_column_widths(self):
        cur_col_width, max_col_width = self.cur_col_width, self.MaxColumnWidth

        for row_index, row in enumerate(self):
            for column_index, cell in enumerate(row):
                size = len(cell) + 3
                col_width = cur_col_width[column_index]
                max_width = max_col_width[column_index]
                if size > col_width:
                    col_width = size
                if max_width >= 0:
                    col_width = min(max_width, col_width)
                cur_col_width[column_index] = col_width

    def genOutput(self):
        return self.lines()

    def _get_separator_row(self, separator):
        columns = []
        for i, width in enumerate(self.cur_col_width):
            if isinstance(separator[i], str):
                column = " " + (width - 1) * separator[i]
            else:
                column = " " + separator[i][:width - 1]
            columns.append(column)
        return "".join(columns)

    def lines(self):
        self._calc_column_widths()
        cur_col_width = self.cur_col_width
        alignment = self.TextAlignment
        ret = []
        wrapper = textwrap.TextWrapper()
        for row_index, row in enumerate(self):
            row_nb = 0
            text_columns = []
            for column_index, cell in enumerate(row):
                align = alignment[column_index]
                width = cur_col_width[column_index]
                wrapper.width = width - 3
                cells = wrapper.wrap(cell)
                for i, c in enumerate(cells):
                    if align & Alignment.Left:
                        cells[i] = " " + c.ljust(width - 1)
                    elif align & Alignment.Right:
                        cells[i] = c.rjust(width)
                    elif align & Alignment.HCenter:
                        cells[i] = c.center(width)
                row_nb = max(len(cells), row_nb)
                text_columns.append(cells)

            text_rows = row_nb * ['']
            for column_index, cells in enumerate(text_columns):
                for i in range(row_nb):
                    if i < len(cells):
                        text_rows[i] = text_rows[i] + cells[i]
                    else:
                        width = cur_col_width[column_index]
                        text_rows[i] = text_rows[i] + width * " "
            ret.extend(text_rows)
            if row_index > 0 and self.row_separator is not None:
                ret.append(self._get_separator_row(self.RowSeparator))

        if self.header_separator is not None:
            ret.insert(1, self._get_separator_row(self.HeaderSeparator))
        return ret

    def str(self):
        return "\n".join(self.lines())

    def __str__(self):
        return self.str()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides Qt styles"""

__all__ = ["setTaurusStyle"]

__docformat__ = 'restructuredtext'

import imp

from taurus.external.qt import Qt


def setTaurusStyle(newStyle):

    app = Qt.QApplication.instance()

    if app is None:
        raise RuntimeError(
            "Must initialize a QApplication before setting style")

    mod = imp.load_module(newStyle, *imp.find_module(newStyle, __path__))

    style = mod.getStyle()
    styleSheet = mod.getStyleSheet()

    if style is not None:
        app.setStyle(style)
    if styleSheet is not None:
        app.setStyleSheet(styleSheet)



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a set of basic Taurus widgets based on QLabel"""

__all__ = ["TaurusLabel"]

__docformat__ = 'restructuredtext'

import operator
import re

# shame of me for importing PyTango!
import PyTango

from taurus.core.taurusbasetypes import TaurusElementType, TaurusEventType
from taurus.external.qt import Qt
from taurus.qt.qtgui.base import TaurusBaseWidget
from taurus.qt.qtgui.base import TaurusBaseController
from taurus.qt.qtgui.base import TaurusScalarAttributeControllerHelper
from taurus.qt.qtgui.base import TaurusConfigurationControllerHelper
from taurus.qt.qtgui.base import updateLabelBackground

_QT_PLUGIN_INFO = {
    'module': 'taurus.qt.qtgui.display',
    'group': 'Taurus Display',
    'icon': "designer:label.png",
}

TaurusModelType = TaurusElementType
EventType = TaurusEventType


class TaurusLabelController(TaurusBaseController):

    StyleSheetTemplate = "border-style: outset; border-width: 2px; border-color: {0}; {1}"

    def __init__(self, label):
        self._text = ''
        self._trimmedText = False
        self._trimPattern = re.compile('<[^<]*>')
        TaurusBaseController.__init__(self, label)

    def _setStyle(self):
        TaurusBaseController._setStyle(self)
        label = self.label()
        # if update as palette
        if self.usePalette():
            label.setFrameShape(Qt.QFrame.Box)
            label.setFrameShadow(Qt.QFrame.Raised)
            label.setLineWidth(1)

    def label(self):
        return self.widget()

    def showValueDialog(self, label):
        Qt.QMessageBox.about(label,  "Full text", self._text)

    def _needsStateConnection(self):
        label = self.label()
        ret = 'state' in (label.fgRole, label.bgRole)
        return ret

    def _updateForeground(self, label):
        fgRole, value = label.fgRole, ''

        # handle special cases (that are not covered with fragment)
        if fgRole.lower() == 'state':
            value = self.state().name
        elif fgRole.lower() in ('', 'none'):
            pass
        else:
            value = label.getDisplayValue(fragmentName=fgRole)
        self._text = text = label.prefixText + value + label.suffixText

        # Checks that the display fits in the widget and sets it to "..." if
        # it does not fit the widget
        self._trimmedText = self._shouldTrim(label, text)
        if self._trimmedText:
            text = "<a href='...'>...</a>"
        label.setText(text)

    def _shouldTrim(self, label, text):
        if not label.autoTrim:
            return False
        text = re.sub(self._trimPattern, '', text)
        font_metrics = Qt.QFontMetrics(label.font())
        size, textSize = label.size().width(), font_metrics.width(text)
        return textSize > size

    def _updateToolTip(self, label):
        if not label.getAutoTooltip():
            return
        toolTip = label.getFormatedToolTip()
        if self._trimmedText:
            toolTip = u"<p><b>Value:</b> %s</p><hr>%s" %\
                      (unicode(self._text, errors='replace'),
                       unicode(str(toolTip), errors='replace'))
        label.setToolTip(toolTip)

    _updateBackground = updateLabelBackground


class TaurusLabelControllerAttribute(TaurusScalarAttributeControllerHelper, TaurusLabelController):

    def __init__(self, label):
        TaurusScalarAttributeControllerHelper.__init__(self)
        TaurusLabelController.__init__(self, label)

    def _setStyle(self):
        TaurusLabelController._setStyle(self)
        label = self.label()
        label.setDynamicTextInteractionFlags(
            Qt.Qt.TextSelectableByMouse | Qt.Qt.LinksAccessibleByMouse)


class TaurusLabelControllerConfiguration(TaurusConfigurationControllerHelper, TaurusLabelController):

    def __init__(self, label):
        TaurusConfigurationControllerHelper.__init__(self)
        TaurusLabelController.__init__(self, label)

    def _setStyle(self):
        TaurusLabelController._setStyle(self)
        label = self.label()
        label.setDynamicTextInteractionFlags(Qt.Qt.NoTextInteraction)


#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
# Design time controllers for label
#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

class TaurusLabelControllerDesignMode(object):

    def _updateLength(self, lcd):
        lcd.setNumDigits(6)

    def getDisplayValue(self, write=False):
        v = self.w_value()
        if not write:
            v = self.value()
        return "%6.2f" % v

    def value(self):
        return 99.99

    def w_value(self):
        return 0.0

    def quality(self):
        return PyTango.AttrQuality.ATTR_VALID

    def state(self):
        return PyTango.DevState.ON

    def _updateToolTip(self, lcd):
        lcd.setToolTip("Some random value for design purposes only")


class TaurusLabelControllerAttributeDesignMode(TaurusLabelControllerDesignMode, TaurusLabelControllerAttribute):

    def __init__(self, label):
        TaurusLabelControllerDesignMode.__init__(self)
        TaurusLabelControllerAttribute.__init__(self, label)


class TaurusLabelControllerConfigurationDesignMode(TaurusLabelControllerDesignMode, TaurusLabelControllerConfiguration):

    def __init__(self, label):
        TaurusLabelControllerDesignMode.__init__(self)
        TaurusLabelControllerConfiguration.__init__(self, label)

    def getDisplayValue(self, write=False):
        return "%6.2f" % -99.99

    def _updateToolTip(self, lcd):
        lcd.setToolTip(
            "Some random configuration value for design purposes only")


_CONTROLLER_MAP = {
    None: None,
    TaurusModelType.Unknown: None,
    TaurusModelType.Attribute: TaurusLabelControllerAttribute,
    TaurusModelType.Configuration: TaurusLabelControllerConfiguration,
}

_DESIGNER_CONTROLLER_MAP = {
    None: TaurusLabelControllerAttributeDesignMode,
    TaurusModelType.Unknown: TaurusLabelControllerAttributeDesignMode,
    TaurusModelType.Attribute: TaurusLabelControllerAttributeDesignMode,
    TaurusModelType.Configuration: TaurusLabelControllerConfigurationDesignMode,
}


class TaurusLabel(Qt.QLabel, TaurusBaseWidget):

    DefaultPrefix = ''
    DefaultSuffix = ''
    DefaultBgRole = 'quality'
    DefaultFgRole = 'rvalue'
    DefaultShowText = True
    DefaultModelIndex = None
    DefaultAutoTrim = True
    DefaultAlignment = Qt.Qt.AlignRight | Qt.Qt.AlignVCenter

    _deprecatedRoles = dict(value='rvalue', w_value='wvalue')

    def __init__(self, parent=None, designMode=False):
        self._prefix = self.DefaultPrefix
        self._suffix = self.DefaultSuffix
        self._bgRole = self.DefaultBgRole
        self._fgRole = self.DefaultFgRole
        self._modelIndex = self.DefaultModelIndex
        self._autoTrim = self.DefaultAutoTrim
        self._modelIndexStr = ''
        self._controller = None
        self._dynamicTextInteractionFlags = True
        name = self.__class__.__name__
        self.call__init__wo_kw(Qt.QLabel, parent)
        self.call__init__(TaurusBaseWidget, name, designMode=designMode)

        self.setAlignment(self.DefaultAlignment)

        self.linkActivated.connect(self.showValueDialog)

        # if we are in design mode there will be no events so we force the
        # creation of a controller object
        if self._designMode:
            self.controllerUpdate()

    def _calculate_controller_class(self):
        ctrl_map = _CONTROLLER_MAP
        if self._designMode:
            ctrl_map = _DESIGNER_CONTROLLER_MAP

        model_type = self.getModelType()
        # ugly workaround to adapt TaurusLabel to tep14 without refactoring
        # TODO: proper refactoring of TaurusValue to supress the Conf API
        if model_type == TaurusModelType.Attribute and self.modelFragmentName:
            model_type = TaurusModelType.Configuration
        ctrl_klass = ctrl_map.get(model_type, TaurusLabelController)
        return ctrl_klass

    def controller(self):
        ctrl = self._controller
        # if there is a controller object and it is not the base controller...
        if ctrl is not None and not ctrl.__class__ == TaurusLabelController:
            return ctrl

        # if there is a controller object and it is still the same class...
        ctrl_klass = self._calculate_controller_class()
        if ctrl_klass is None:
            return None
        elif ctrl.__class__ == ctrl_klass:
            return ctrl

        self._controller = ctrl = ctrl_klass(self)
        return ctrl

    def controllerUpdate(self):
        ctrl = self.controller()
        if ctrl is not None:
            ctrl.update()

    def showValueDialog(self, *args):
        ctrl = self.controller()
        if ctrl is not None:
            ctrl.showValueDialog(self)

    def resizeEvent(self, event):
        #    # recheck the display every time we resize to make sure the text should
        #    # become trimmed or not
        if not getattr(self, '_inResize', False):
            self._inResize = True
            self.controllerUpdate()
            self._inResize = False
        Qt.QLabel.resizeEvent(self, event)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TaurusBaseWidget overwriting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def handleEvent(self, evt_src, evt_type, evt_value):
        ctrl = self.controller()
        if ctrl is not None:
            ctrl.handleEvent(evt_src, evt_type, evt_value)

    def isReadOnly(self):
        return True

    def setModel(self, m):
        # force to build another controller
        self._controller = None
        TaurusBaseWidget.setModel(self, m)
        if self.modelFragmentName:
            self.setFgRole(self.modelFragmentName)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT property definition
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getModelIndexValue(self):
        return self._modelIndex

    def getModelIndex(self):
        return self._modelIndexStr

    def setModelIndex(self, modelIndex):
        mi = str(modelIndex)
        if len(mi) == 0:
            self._modelIndex = None
        else:
            try:
                mi_value = eval(str(mi))
            except:
                return
            if type(mi_value) == int:
                mi_value = mi_value,
            if not operator.isSequenceType(mi_value):
                return
            self._modelIndex = mi_value
        self._modelIndexStr = mi
        self.controllerUpdate()

    def getModelMimeData(self):
        mimeData = TaurusBaseWidget.getModelMimeData(self)
        mimeData.setText(self.text())
        return mimeData

    def resetModelIndex(self):
        self.setModelIndex(self.DefaultModelIndex)

    def getBgRole(self):
        return self._bgRole

    def setBgRole(self, bgRole):
        self._bgRole = str(bgRole).lower()
        self.controllerUpdate()

    def resetBgRole(self):
        self.setBgRole(self.DefaultBgRole)

    def getFgRole(self):
        return self._fgRole

    def setFgRole(self, fgRole):
        # warn about deprecated roles
        role = self._deprecatedRoles.get(fgRole, fgRole)
        if fgRole != role:
            self.deprecated(rel='4.0', dep='setFgRole(%s)' % fgRole,
                            alt='setFgRole(%s)' % role)
        self._fgRole = str(role)
        self.controllerUpdate()

    def resetFgRole(self):
        self.setFgRole(self.DefaultFgRole)

    def getPrefixText(self):
        return self._prefix

    def setPrefixText(self, prefix):
        self._prefix = str(prefix)
        self.controllerUpdate()

    def resetPrefixText(self):
        self.setPrefixText(self.DefaultPrefix)

    def getSuffixText(self):
        return self._suffix

    def setSuffixText(self, suffix):
        self._suffix = str(suffix)
        self.controllerUpdate()

    def resetSuffixText(self):
        self.setSuffixText(self.DefaultSuffix)

    def setAutoTrim(self, trim):
        self._autoTrim = trim
        self.controllerUpdate()

    def setDynamicTextInteractionFlags(self, flags):
        if self.hasDynamicTextInteractionFlags():
            Qt.QLabel.setTextInteractionFlags(self, flags)

    def hasDynamicTextInteractionFlags(self):
        return self._dynamicTextInteractionFlags

    def setTextInteractionFlags(self, flags):
        Qt.QLabel.setTextInteractionFlags(self, flags)
        self._dynamicTextInteractionFlags = False

    def resetTextInteractionFlags(self):
        Qt.QLabel.resetTextInteractionFlags(self)
        self.dynamicTextInteractionFlags = True

    def getAutoTrim(self):
        return self._autoTrim

    def resetAutoTrim(self):
        self.setAutoTrim(self.DefaultAutoTrim)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        d = TaurusBaseWidget.getQtDesignerPluginInfo()
        d.update(_QT_PLUGIN_INFO)
        return d

    #: This property holds the unique URI string representing the model name
    #: with which this widget will get its data from. The convention used for
    #: the string can be found :ref:`here <model-concept>`.
    #:
    #: In case the property :attr:`useParentModel` is set to True, the model
    #: text must start with a '/' followed by the attribute name.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusBaseWidget.getModel`
    #:     * :meth:`TaurusLabel.setModel`
    #:     * :meth:`TaurusBaseWidget.resetModel`
    #:
    #: .. seealso:: :ref:`model-concept`
    model = Qt.pyqtProperty("QString", TaurusBaseWidget.getModel, setModel,
                            TaurusBaseWidget.resetModel)

    #: This property holds whether or not this widget should search in the
    #: widget hierarchy for a model prefix in a parent widget.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusBaseWidget.getUseParentModel`
    #:     * :meth:`TaurusBaseWidget.setUseParentModel`
    #:     * :meth:`TaurusBaseWidget.resetUseParentModel`
    #:
    #: .. seealso:: :ref:`model-concept`
    useParentModel = Qt.pyqtProperty("bool", TaurusBaseWidget.getUseParentModel,
                                     TaurusBaseWidget.setUseParentModel,
                                     TaurusBaseWidget.resetUseParentModel)

    #: This property holds the index inside the model value that should be
    #: displayed
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.getModelIndex`
    #:     * :meth:`TaurusLabel.setModelIndex`
    #:     * :meth:`TaurusLabel.resetModelIndex`
    #:
    #: .. seealso:: :ref:`model-concept`
    modelIndex = Qt.pyqtProperty(
        "QString", getModelIndex, setModelIndex, resetModelIndex)

    #: This property holds a prefix text
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.getPrefixText`
    #:     * :meth:`TaurusLabel.setPrefixText`
    #:     * :meth:`TaurusLabel.resetPrefixText`
    prefixText = Qt.pyqtProperty("QString", getPrefixText, setPrefixText,
                                 resetPrefixText, doc="prefix text")

    #: This property holds a suffix text
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.getSuffixText`
    #:     * :meth:`TaurusLabel.setSuffixText`
    #:     * :meth:`TaurusLabel.resetSuffixText`
    suffixText = Qt.pyqtProperty("QString", getSuffixText, setSuffixText,
                                 resetSuffixText, doc="suffix text")

    #: This property holds the foreground role (the text).
    #: Valid values are:
    #:
    #:     #. ''/'None' - no value is displayed
    #:     #. 'value' - the value is displayed
    #:     #. 'w_value' - the write value is displayed
    #:     #. 'quality' - the quality is displayed
    #:     #. 'state' - the device state is displayed
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.getFgRole`
    #:     * :meth:`TaurusLabel.setFgRole`
    #:     * :meth:`TaurusLabel.resetFgRole`
    fgRole = Qt.pyqtProperty("QString", getFgRole, setFgRole,
                             resetFgRole, doc="foreground role")

    #: This property holds the background role.
    #: Valid values are ''/'None', 'quality', 'state'
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.getBgRole`
    #:     * :meth:`TaurusLabel.setBgRole`
    #:     * :meth:`TaurusLabel.resetBgRole`
    bgRole = Qt.pyqtProperty("QString", getBgRole, setBgRole,
                             resetBgRole, doc="background role")

    #: This property holds the
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.getAutoTrim`
    #:     * :meth:`TaurusLabel.setAutoTrim`
    #:     * :meth:`TaurusLabel.resetAutoTrim`
    autoTrim = Qt.pyqtProperty("bool", getAutoTrim, setAutoTrim,
                               resetAutoTrim, doc="auto trim text")

    #: This property holds the
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.isDragEnabled`
    #:     * :meth:`TaurusLabel.setDragEnabled`
    #:     * :meth:`TaurusLabel.resetDragEnabled`
    dragEnabled = Qt.pyqtProperty("bool", TaurusBaseWidget.isDragEnabled,
                                  TaurusBaseWidget.setDragEnabled,
                                  TaurusBaseWidget.resetDragEnabled,
                                  doc="enable dragging")

    #: Specifies how the label should interact with user input if it displays
    #: text.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusLabel.textInteractionFlags`
    #:     * :meth:`TaurusLabel.setTextInteractionFlags`
    #:     * :meth:`TaurusLabel.resetTextInteractionFlags
    try:
        textInteractionFlags = Qt.pyqtProperty(Qt.Qt.TextInteractionFlag,
                                               Qt.QLabel.textInteractionFlags,
                                               setTextInteractionFlags,
                                               resetTextInteractionFlags,
                                               doc="Specifies how the label should interact with user input if it displays text.")
    except TypeError:  # Old PyQt4 version only accept strings for the type arg
        textInteractionFlags = Qt.pyqtProperty("int",
                                               Qt.QLabel.textInteractionFlags,
                                               setTextInteractionFlags,
                                               resetTextInteractionFlags,
                                               doc="Specifies how the label should interact with user input if it displays text.")


def demo():
    "Label"
    import demo.tauruslabeldemo
    return demo.tauruslabeldemo.main()


def main():
    import sys
    import taurus.qt.qtgui.application
    Application = taurus.qt.qtgui.application.TaurusApplication

    app = Application.instance()
    owns_app = app is None

    if owns_app:
        import taurus.core.util.argparse
        parser = taurus.core.util.argparse.get_taurus_parser()
        parser.usage = "%prog [options] <full_attribute_name(s)>"
        app = Application(sys.argv, cmd_line_parser=parser,
                          app_name="Taurus label demo", app_version="1.0",
                          org_domain="Taurus", org_name="Tango community")

    args = app.get_command_line_args()

    if len(args) == 0:
        w = demo()
    else:
        models = map(str.lower, args)

        w = Qt.QWidget()
        layout = Qt.QGridLayout()
        w.setLayout(layout)
        for model in models:
            label = TaurusLabel()
            label.model = model
            layout.addWidget(label)
    w.show()

    if owns_app:
        sys.exit(app.exec_())
    else:
        return w

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

import functools

from taurus import Device, Attribute  # , Authority
from taurus.test import insertTest


class TaurusModelEqualityTestCase(object):
    """Base class for taurus model equality testing."""

    def modelsEqual(self, models, class_, equal=True):
        """A helper method to create tests that checks equality (or inequality)
        of taurus objects e.g. TaurusAttribute.

        :param models: (seq<str>): a sequence of two taurus models
        :param class_: (function) model factory function
        :param equal: (bool) If True, check equality. Else check inequality
        """
        name1, name2 = models
        obj1 = class_(name1)
        obj2 = class_(name2)
        if equal:
            msg = 'models for %s and %s are not equal (they should)' % \
                  (name1, name2)
            self.assertIs(obj1, obj2, msg)
        else:
            msg = 'models for %s and %s are equal (they should not)' % \
                  (name1, name2)
            self.assertIsNot(obj1, obj2, msg)


testDeviceModelEquality = functools.partial(insertTest,
                                            helper_name='modelsEqual',
                                            class_=Device,
                                            test_method_name='testDeviceModelEquality')

testAttributeModelEquality = functools.partial(insertTest,
                                               helper_name='modelsEqual',
                                               class_=Attribute,
                                               test_method_name='testAttributeModelEquality')
